/*
 * guid.c : automatic guiding
 *
 * 2003.05.28 -
 * Time-stamp: <2005-01-12 00:10:03 dimm>
 */

#include <math.h>
#include <gtk/gtk.h>

#include "protocol.h"
#include "seemon2.h"

/* check if guinding is needed
 * v.0.4.1 2003.05.28
 * when this function is called,
 * tracking is on (mseemon.is_tracking == 1)
 */
void check_guid(void) {
  int i;
  int flg = 0;

  /* check positions */
  for (i = 0; i < 2; i++) {
    if (mseemon.star_center[i].cx < boundx1 ||
	mseemon.star_center[i].cy < boundy1 ||
	mseemon.star_center[i].cx > boundx2 ||
	mseemon.star_center[i].cy > boundy2) 
      flg++;
  }

  if (flg > 0) {
    mseemon.is_guiding = 1;
  }
}

/* start guiding */
/* 2003.05.28
 * 2003.05.29 update
 * 2004.12.01 update
 */
/* directions:
 *        W(8)
 *        |
 * N(1)--   --S(4)
 *        |
 *        E(2)
 */
void start_guid(void) {
  int dx, dy, itime, teldir, diff;
  int flgx = 0, flgy = 0;
  pixcoord m1, center;
  double dist, dtime;
  char str[10];
  myTime mt;
  double d, tmjd, lst, alt, calt;
  int factor = 1;

  center.cx = IMG_WID/2;
  center.cy = IMG_HGT/2;

  /* first, confirm all motors have stopped */
  StopSlewAll();

  /* set coords */
  m1.cx = (mseemon.star_center[0].cx + mseemon.star_center[1].cx) /2;
  m1.cy = (mseemon.star_center[0].cy + mseemon.star_center[1].cy) /2;

  dx = m1.cx - center.cx;
  dy = m1.cy - center.cy;

  /* get altitude */
  get_localtime(&(mseemon.ctime));
  get_time(&mt);
  d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
  tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
  lst = get_lst(tmjd, msite.slon);
  get_azel(mseemon.selnum, lst);
  calt = cos(obj_edb[mseemon.selnum]->s_alt);
  if (calt < 0.1) calt = 0.1;
  alt = obj_edb[mseemon.selnum]->s_alt * 180. / PI;

  /* if high altitude, extend motion */
  if (alt > mseemon.alt_extra) {
    factor = mseemon.high_extra;
    #ifdef DEBUG
    printf("# extra motion: alt=%6.2f factor=%d\n", alt, factor);
    #endif
  }

  /* motion for x direction */
  if (dx > 2 * boundx1 || dx < -2 * boundx1) {
    /* need motion */
    flgx = 1;

    /* set slew rate to CENTER */
    //    mseemon.tel_step = CENTER;
    //    SetRate(mseemon.tel_step);
    /* test. RC0 is 12/64 of RC1 speed */
    SetRateCenter0();
    mseemon.tel_step = CENTER; /* added by iwata v0.5.10 20040810 */

    switch(mseemon.dimm_id) {
    case 1:
      /* direction */
      if (dx < 0) {
	teldir = SOUTH;
	dx *= -1;
      } else {
	teldir = NORTH;
      }
    default:
      /* direction */
      if (dx < 0) {
	teldir = EAST;
	dx *= -factor;
      } else {
	teldir = WEST;
	dx *= factor;
      }
    }

    dist = dx * PIXSCALEX; /* in arcsec */
    /* set duration of motion.
     *  guide speed is 15.0417"/sec.
     *  (testing: 2003.08.16)
     */
    /* time in usec */
    /* CENTER is 16xsiderial motion */
    dtime = dist * 1000000 / (2 * 16 *15.0417);
    itime = (int)dtime;

    /* if star position did not change significantly, 
     * do additional motion 
     * added v0.6.8 20050110
     */
    if (mseemon.prevgdiff[1].cx > -900) {
      diff = mseemon.prevgdiff[0].cx - mseemon.prevgdiff[1].cx;
      if (diff < 5 && diff > -5) {
	itime += 1000000;
	if (fout != NULL) {
	  fprintf(fout, "# guidx: additional time of 1.0sec.\n");
	}
	#ifdef DEBUG
	printf("# guidx: additional time of 0.5sec.\n");
	#endif
      }
    }
    mseemon.prevgdiff[1].cx = mseemon.prevgdiff[0].cx;
    mseemon.prevgdiff[0].cx = dx;

    #ifdef DEBUG
    printf("# dx:%3d itime=%3d teldir=%d\n", dx, itime, teldir);
    #endif

    StartSlew(teldir);
    usleep(itime);
    StopSlew(teldir);

    if (fout != NULL) {
      /* append to log */
      get_localtime(&(mseemon.ctime));

      sprintf(str, "guide %02d:%02d:%02d", 
	      mseemon.ctime.hour,
	      mseemon.ctime.min,
	      mseemon.ctime.sec);
      fprintf(fout, "# %s xdir:%d dx:%3d time:%2d\n",
	      str, teldir, dx, itime);
    }
  }

  /* motion for y direction */
  if (dy > 2 * boundy1 || dy < -2 * boundy1) {
    /* need motion */
    flgy = 1;
    SetRateCenter0();
    mseemon.tel_step = CENTER; /* added by iwata v0.5.10 20040810 */

    switch(mseemon.dimm_id) {
    case 1:
      /* direction */
      if (dy < 0) {
	teldir = EAST;
	dy *= -factor;
      } else {
	teldir = WEST;
	dy *= factor;
      }
    default:
      /* direction */
      if (dy < 0) {
	teldir = SOUTH;
	dy *= -1;
      } else {
	teldir = NORTH;
      }
    }

    dist = dy * PIXSCALEY; /* in arcsec */
    //    dtime = dist / 15.0417; /* guide speed is 15.0417"/sec */
    /* time in usec */
    /* CENTER is 16xsiderial motion */
    dtime = dist * 1000000 / (8 * calt * 16 *15.0417);
    itime = (int)dtime;

    /* if star position did not change significantly, 
     * do additional motion 
     * added v0.6.8 20050110
     */
    if (mseemon.prevgdiff[1].cy > -900) {
      diff = mseemon.prevgdiff[0].cy - mseemon.prevgdiff[1].cy;
      if (diff < 5 && diff > -5) {
	itime += 500000;
	if (fout != NULL) {
	  fprintf(fout, "# guidy: additional time of 0.5sec.\n");
	}
	#ifdef DEBUG
	printf("# guidy: additional time of 0.5sec.\n");
	#endif
      }
    }
    mseemon.prevgdiff[1].cy = mseemon.prevgdiff[0].cy;
    mseemon.prevgdiff[0].cy = dy;

    #ifdef DEBUG
    printf("# dy:%3d itime=%3d teldir=%d\n", dy, itime, teldir);
    #endif

    StartSlew(teldir);
    usleep(itime);
    StopSlew(teldir);

    if (fout != NULL) {
      /* append to log */
      get_localtime(&(mseemon.ctime));

      sprintf(str, "guide %02d:%02d:%02d", 
	      mseemon.ctime.hour,
	      mseemon.ctime.min,
	      mseemon.ctime.sec);
      fprintf(fout, "# %s ydir:%d dy:%3d time:%2d ntry2:%d\n",
	      str, teldir, dy, itime, mseemon.ntry2);
    }
  }

  /* no more guiding needed ? */
  if (flgx == 0 && flgy == 0) {
    mseemon.is_guiding = 0;
    #ifdef DEBUG
    printf("found a star at center successfully .\n");
    fprintf(fout, "# found a star at center successfully .\n");
    #endif
    mseemon.ntry2 = 0; /* reset. v0.6.6 20041210 */
    mseemon.prevgdiff[0].cx = -999;
    mseemon.prevgdiff[0].cy = -999;
    mseemon.prevgdiff[1].cx = -999;
    mseemon.prevgdiff[1].cy = -999;
  } else {
    mseemon.ntry2++;
  }

  /* sleep for a while */
  #ifdef DEBUG
  printf("sleeping for a while...\n");
  #endif
  sleep(5);

  if (fout != NULL) fflush(fout);
}
