/* 
 * 2002.10.18 -
 * Time-stamp: <2005-02-11 04:41:33 dimm>
*/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "seemon2.h"
#include "protocol.h" /* from xmtel */

int main( int   argc,
          char *argv[] )
{
  int i, ret, opt;
  Obj *cobj;
  double diam, sep;
  char str[30];

  /* GtkWidget is the storage type for widgets */
  GtkWidget *window;
  GtkWidget *window_cap;    /* capture window */
  GtkWidget *window_telctl; /* telescope control */
  GtkWidget *menubar;

  GtkWidget *button, *frame, *label;
  GtkWidget *vbox1, *vbox2;
  GtkWidget *hbox1, *hbox2;
  GtkWidget *entry;

  /* This is called in all GTK applications. Arguments are parsed
   * from the command line and are returned to the application. */
  gtk_init (&argc, &argv);

  mseemon.autoflg = 0;
  while ((opt = getopt(argc, argv, "a")) != -1) {
    switch (opt) {
    case 'a':
      mseemon.autoflg = 1;
      printf("# seemon2: auto-starting mode.\n");
      break;
    }
  }

  /* setup object data */
  //  strcpy(edbpath, "/usr/local/share/seemon2/sample.edb");
  strcpy(edbpath, "/usr/local/share/seemon2/bsc_v10_20.edb");
  mseemon.objnum = read_edb(edbpath);
  mseemon.selnum = -1; /* no selection */

  /* setup site info */
  /* oao */
  msite.slon = 133.59667; /* 8h 54m 23.2s, +34d 34m 26s */
  msite.slat = 34.573888; /* corrected on v.0.5.6 */
  msite.lt = 9;

  /* set defaults */
  /* timer for reload image */
  timertag = -1;
  /* timer for command-check */
  timertag_comm = -1;
  /* interval for setting sky */
  mseemon.time_peak = 10;
  mseemon.cnt_peak = 0;
  mseemon.peak_average = 0;
  mseemon.n_average = 0;
  /* dimm telescope v0.5.11 20041102 */
  mseemon.dimm_id = 2; /* will be read from conf file */

  /* interval for redraw v0.3.0, 2003.05.16 */
  //  mseemon.time_redraw = 5;
  /*  redraw_count = 0; */
  mseemon.time_reload1 = DEF_TIMER1;
  mseemon.time_reload2 = DEF_TIMER2;
  mseemon.time_reload = mseemon.time_reload1;
  mseemon.clip = CLIP; /* factor for clipping, added ver.0.2.6 */

  /* default I.I.CCD shutter interval v0.5.12 2004.11.18 */
  mseemon.time_iishutter0 = SHUTTER_INTV; /* will be read from conf file */
  mseemon.time_iishutter = mseemon.time_iishutter0;

  /* default command-check interval, added v0.6.2 */
  mseemon.time_reload_comm = DEF_TIMER3;

  mseemon.finder_star_center.cx = -1;
  mseemon.finder_star_center.cy = -1;
  mseemon.finder_peakval = -1;
  mseemon.finder_dist_threshold = 10;
  mseemon.finder_dist = 1000;
  mseemon.finder_dist_prev = 1000;
  mseemon.min_peak = MIN_PEAK;
  mseemon.prevgdiff[0].cx = -999;
  mseemon.prevgdiff[0].cy = -999;
  mseemon.prevgdiff[1].cx = -999;
  mseemon.prevgdiff[1].cy = -999;

  mseemon.endtime = 28.5; /* end time of operation (localtime), will be read from config file */
  mseemon.rainflg = -1;
  mseemon.humidity = -1;

  mseemon.alt_limit1 = 25.0; /* low limit for selecting star, will be read from config file */
  mseemon.alt_limit2 = 25.0; /* low limit for tracking star, will be read from config file */


  /* following 2 parameters will be read from config file */
  mseemon.high_extra = HIGH_EXTRA; /* factor for longer motion for high alt */
  mseemon.alt_extra  = ALT_EXTRA;

  /* step for telescope motion in arcsec */
  mseemon.tel_step = SLEW;
  /* set path of .ppm files. v0.4.0u */
  set_ppm_path();

  /* set status */
  /* not connected, not tracking, not recording */
  mseemon.tel_stat = 0;
  mseemon.tel_step = 2; /* default tel ctl step is center */
  mseemon.is_capturing = 0;
  mseemon.is_tracking = 0;
  mseemon.is_recording = 0;
  mseemon.is_guiding = 0;
  mseemon.is_pointing = 0;
  mseemon.is_sleeping = 0;
  mseemon.sleep_length = 0;
  mseemon.motionflg = 0;
  mseemon.image_dist = 0.;
  mseemon.flg_reload = 0; /* flag for reloading */
  mseemon.ntry1 = 0;
  mseemon.ntry2 = 0;

  /* fraction of fov for guiding */
  mseemon.bound = 0.1;

  /* operation mode v.0.4.6 */
  mseemon.opmode = 0; /* manual operation */
  mseemon.op_ssmode = 1; /* star selection mode v0.5.4 */

  /* guide function is on by default. v.0.4.7 */
  mseemon.guideflg = 1;

  /* set boundaries for guinding. v.0.4.1 */
  boundx1 = (int)(IMG_WID * mseemon.bound);
  boundx2 = IMG_WID - boundx1;
  boundy1 = (int)(IMG_HGT * mseemon.bound);
  boundy2 = IMG_HGT - boundy1;
  #ifdef DEBUG
  printf("boundaries for guiding:1: %d %d 2: %d %d\n", boundx1, boundy1, boundx2, boundy2);
  #endif

  /* default rectangular image buffer */
  for (i=0; i < 2; i++) {
    mseemon.star_rect[i].cx = RECT_WID;
    mseemon.star_rect[i].cy = RECT_HGT;
    mseemon.cg_rect[i].cx = CG_RECT;
    mseemon.cg_rect[i].cy = CG_RECT;
    pImRect[i] = NULL;
    flg_set_rect[i] = 0;
    mseemon.cg[i].x = 0.;
    mseemon.cg[i].y = 0.;
    mseemon.peak[i].cx = 0;
    mseemon.peak[i].cy = 0;
    mseemon.peakval[i] = 0.;
  }

  /* init variables in get_cg.c */
  bgthr = 1.; /* default background threshold */
  cgfactor = 1.;
  n_calc_r0 = 1;
  /*  n_totalsample = 0; /* commented out v0.5.10 20040802 iwata */
  Lambda = LAMBDA;
  diam = DIAM;	/* diameter of apertures in unit of metre */
  sep = SEP;	/* separation of apertures in unit of metre */
  r0k.x = 2 * Lambda * Lambda * ( 0.179 * pow(diam, -1./3.) - 0.145 * pow(sep, -1./3.));
  r0k.y = 2 * Lambda * Lambda * ( 0.179 * pow(diam, -1./3.) - 0.0968 * pow(sep, -1./3.));

  /* scale of one pixel in radian. */
  scalex = PIXSCALEX * PI / (180*60*60);
  scaley = PIXSCALEY * PI / (180*60*60);
  Sigma.x = 0.;
  Sigma.y = 0.;
  NSample = 50;

  for (i = 0; i < NSample; i++) {
    mseemon.prev_diffcg[i].x = 0.;
    mseemon.prev_diffcg[i].y = 0.;
  }

  /* read configuration file */
  ret = read_config(PREFPATH); /* in prefs.c */
  if (ret == EXIT_FAILURE) {
    printf("# ERROR: failed in reading from %s.\n", PREFPATH);
    if (mseemon.autoflg == 0) {
      exit(EXIT_FAILURE);
    }
  }

  /* socket connection */
  fd_socket = -1;
  if ((ret = connect_socket(socket_server)) == EXIT_SUCCESS) {
    printf("connected to %s.\n", socket_server);
  } else {
    /* alert dialog */
    sprintf(str, "unable to connect to %s", socket_server);
    fd_socket=-1;
    mydialog(str);
  }


  /* intialize log file */
  setoutputfile();

  /* finder target position file */
  /* added v0.5.6 2004.02.20 */
  strcpy(tgtpath, FDRTGTPATH);
  get_finder_target_pos();  /* in prefs.c */

  /* Create windows */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  window_cap = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  window_telctl = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  menubar = init_main_menu(window);
    
  /* set titles */
  gtk_window_set_title (GTK_WINDOW (window), "Seemon2: Main");
  gtk_window_set_title (GTK_WINDOW (window_cap), "Seemon2: Capture");
  gtk_window_set_title (GTK_WINDOW (window_telctl), "Seemon2: Telescope");

  /* Here we just set a handler for delete_event that immediately
   * exits GTK. */
  g_signal_connect (G_OBJECT (window), "delete_event",
		    G_CALLBACK (delete_event), NULL);

  /* Sets the border width of the window. */
  gtk_container_set_border_width (GTK_CONTAINER (window), 2);
  gtk_container_set_border_width (GTK_CONTAINER (window_cap), 10);
  gtk_container_set_border_width (GTK_CONTAINER (window_telctl), 10);

  /*****
   * Making contents of Main Window
   *****/
  vbox1 = gtk_vbox_new (FALSE, 5);
  vbox2 = gtk_vbox_new (FALSE, 0);
  hbox1 = gtk_hbox_new (FALSE, 0);
  hbox2 = gtk_hbox_new (FALSE, 0);

  /* add menubar */
  gtk_box_pack_start (GTK_BOX(vbox1), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);

  gtk_container_add (GTK_CONTAINER (window), vbox1);
  gtk_box_pack_start (GTK_BOX(vbox1), vbox2, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox1), hbox1, TRUE, TRUE, 0);

  /* operation mode display */
  frame = gtk_frame_new ("Op. Mode");
  entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry), "Manual");

  button = gtk_button_new_with_label("Switch");
  //  gtk_box_pack_start(GTK_BOX(hbox2), button, TRUE, TRUE, 0);
  g_signal_connect(G_OBJECT(button), "clicked", 
		   G_CALLBACK(toggle_opbutton),NULL);
  gtk_container_add(GTK_CONTAINER (frame), button);
  gtk_box_pack_start(GTK_BOX (hbox2), frame, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);

  gtk_widget_show (button);
  gtk_widget_show (frame);
  gtk_widget_show (entry);
  gtk_widget_show (hbox2);

  ent_opmode = NULL;
  ent_opmode = (GtkWidget*)entry;


  /* guide mode display */
  hbox2 = gtk_hbox_new (FALSE, 0);
  frame = gtk_frame_new ("Star-tracking (Guide) ");
  entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry), "ON ");

  button = gtk_button_new_with_label("Switch");
  g_signal_connect(G_OBJECT(button), "clicked", 
		   G_CALLBACK(toggle_guidebutton),NULL);
  gtk_container_add(GTK_CONTAINER (frame), button);
  gtk_box_pack_start(GTK_BOX (hbox2), frame, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);

  gtk_widget_show (button);
  gtk_widget_show (frame);
  gtk_widget_show (entry);
  gtk_widget_show (hbox2);

  ent_guidemode = NULL;
  ent_guidemode = (GtkWidget*)entry;

  /* telescope status text entry */
  hbox2 = gtk_hbox_new (FALSE, 0);
  frame = gtk_frame_new ("Telescope");
  label = gtk_label_new ("status");
  entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry), "disconnected");

  gtk_container_add(GTK_CONTAINER (frame), label);
  gtk_box_pack_start(GTK_BOX (hbox2), frame, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);

  gtk_widget_show (label);
  gtk_widget_show (frame);
  gtk_widget_show (entry);
  gtk_widget_show (hbox2);

  ent_tel_stat1 = NULL;
  ent_tel_stat1 = (GtkWidget*)entry;

  /* telescope r.a. and dec. */
  hbox2 = gtk_hbox_new (FALSE, 0);
  label = gtk_label_new ("    Tel R.A.");
  entry = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
  gtk_box_pack_start(GTK_BOX (hbox2), label, TRUE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, FALSE, 0);
  gtk_widget_show (label);
  gtk_widget_show (entry);
  ent_tel_ra = (GtkWidget*)entry;

  label = gtk_label_new ("Dec.");
  entry = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
  gtk_box_pack_start(GTK_BOX (hbox2), label, TRUE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);
  gtk_widget_show (label);
  gtk_widget_show (entry);
  gtk_widget_show (hbox2);
  ent_tel_dc = (GtkWidget*)entry;

  /* target r.a. and dec. */
  hbox2 = gtk_hbox_new (FALSE, 0);
  label = gtk_label_new ("Target R.A.");
  entry = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
  gtk_box_pack_start(GTK_BOX (hbox2), label, TRUE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, FALSE, 0);
  gtk_widget_show (label);
  gtk_widget_show (entry);
  ent_trg_ra = (GtkWidget*)entry;

  label = gtk_label_new ("Dec.");
  entry = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
  gtk_box_pack_start(GTK_BOX (hbox2), label, TRUE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);
  gtk_widget_show (label);
  gtk_widget_show (entry);
  gtk_widget_show (hbox2);
  ent_trg_dc = (GtkWidget*)entry;

  /* selected object text entry */
  hbox2 = gtk_hbox_new (FALSE, 0);
  frame = gtk_frame_new ("Selected Object");
  label = gtk_label_new ("Name");
  entry = gtk_entry_new();
  gtk_entry_set_text (GTK_ENTRY (entry), "None");

  gtk_container_add(GTK_CONTAINER (frame), label);
  gtk_box_pack_start(GTK_BOX (hbox2), frame, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX (hbox2), entry, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);

  gtk_widget_show (label);
  gtk_widget_show (frame);
  gtk_widget_show (entry);
  gtk_widget_show (hbox2);

  ent_selobj1 = entry;


  /* telescope control buttons */
  hbox2 = gtk_hbox_new (FALSE, 0);
  /* connetct/disconnect telescope */
  button = gtk_button_new_with_label ("Connect to Telescope");
  gtk_box_pack_start (GTK_BOX(hbox2), button, TRUE, TRUE, 0);
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_connect_telescope), NULL);
  gtk_widget_show (button);
  but_tel_conn = button;

  /* sleep/wake telescope */
  button = gtk_button_new_with_label ("Sleep Telescope");
  gtk_box_pack_start (GTK_BOX(hbox2), button, TRUE, TRUE, 0);
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_sleep_telescope), NULL);
  gtk_widget_show (button);
  but_tel_sleep = button;

  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);
  gtk_widget_show (hbox2);

  hbox2 = gtk_hbox_new (FALSE, 0);
  /* point telescope */
  button = gtk_button_new_with_label ("Point Telescope");
  gtk_box_pack_start (GTK_BOX(hbox2), button, TRUE, TRUE, 0);
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_point_telescope), NULL);
  gtk_widget_show (button);
  /* telescope control */
  button = gtk_button_new_with_label ("Telescope Control");
  gtk_box_pack_start (GTK_BOX(hbox2), button, TRUE, TRUE, 0);
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_open_window), (gpointer) window_telctl);
  gtk_widget_show (button);

  gtk_box_pack_start (GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);
  gtk_widget_show (hbox2);

  gtk_widget_show (vbox2);


  /* button for capture window */
  button = gtk_button_new_with_label ("capture window");
  /* Now when the button is clicked, we call the "callback" function
   * with a pointer to "button 1" as its argument */
  // g_print("%p\n", window_cap); // debug
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_open_window_cap), (gpointer) window_cap);

  /* Instead of gtk_container_add, we pack this button into the invisible
   * box, which has been packed into the window. */
  gtk_box_pack_start (GTK_BOX(hbox1), button, TRUE, TRUE, 0);

  gtk_widget_show (button);
    
  /* button for object dialog */
  button = gtk_button_new_with_label ("object");
  g_signal_connect (G_OBJECT(button), "clicked", 
  		    G_CALLBACK (init_obj_dialog), NULL);
  gtk_box_pack_start(GTK_BOX (hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  /* Do these same steps again to create a second button */
  button = gtk_button_new_with_label ("quit");

  /* Call the same callback function with a different argument,
   * passing a pointer to "button 2" instead. */
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (delete_event), NULL);
  
  gtk_box_pack_start(GTK_BOX (hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);
  gtk_widget_show (hbox1);

  gtk_widget_show (vbox1);
  gtk_widget_show (window);


  /* making contents of capture window */
  init_cap_window(window_cap);
  window_capture = window_cap;

  /* making contents of telescope control dialog */
  init_telctl_window(window_telctl);

  /* start command-check timer */
  start_comm_timer(); /* in command.c */

  printf("now starting seemon2..\n");

  if (fd_socket >= 0) {
    printf("setting IICCD shutter time as %d usec..\n", mseemon.time_iishutter);
    set_iishutter(mseemon.time_iishutter);
    printf("  ...done.\n");
  }

  for (i = 0; i < mseemon.objnum; i++) {
    mseemon.selnum = i;
    cobj = obj_edb[mseemon.selnum];
    mseemon.tel_ra = cobj->s_ra;
    mseemon.tel_dc = cobj->s_dec;
    build_msg();
  }

  /* if auto-mode, connect to the telescope, check dome status 
   * and start obs. */
  if (mseemon.autoflg) {
    ret = auto_startup(); /* in telctl_automatic.c */
  }

  /* Rest in gtk_main and wait for the fun to begin! */
  gtk_main ();

  if (fout != NULL) closeoutputfile();
  
  gtk_main_quit();

  return(EXIT_SUCCESS);
}

