/* 2002.10.26-
 * Time-stamp: <2005-04-20 17:11:32 dimm>
 */
#include <stdio.h>

#include "P_.h"
#include "circum.h"

#ifndef PI
#define	PI		3.141592653589793
#endif

#define IMG_HGT 480
#define IMG_WID 720
#define RECT_HGT  50
#define RECT_WID  80
#define VERSION "0.7.2"
#define QPPMPATH "/mnt/ramdisk/seemon2/"     /* quick look image */
//#define PPMPATH "/usr/local/share/seemon2/"
#define PPMPATH "/mnt/ramdisk/seemon2/"
#define COMMANDFILE "/mnt/ramdisk/seemon2/command.txt" /* v0.6.2 20041126 iwata */
#define FDRTGTPATH "/usr/local/share/seemon2/tgt.txt" /* finder target position. added v0.5.6 2004.02.20 */
#define PREFPATH   "/usr/local/share/seemon2/seemon2.conf" /* configuration file. v0.6.4 20041203 iwata */
#define N_CALC_R0 100
//#define LOGPATH "/import/athelas/data" /* location of to save log files. added v0.5.9 2004.06.28 (Hattori) */
//#define LOGPATH "./" /* location of to save log files. added v0.5.9 2004.06.28 (Hattori) */
#define LOGPATH "/usr/local/share/seemon2/" /* location of to save log files v0.6.3 20041130 iwata */

#define MIN_PEAK 30 /* min. peak value for judging star is within an image */
#define FWHM_MAX 10 /* max of valid fwhm */
#define FNDR_MIN_PEAK 50 /* min. peak value for judging star is within a finder image v.0.5.2 */
#define PEAK_MAX 190 /* max of valid peak value. v0.5.7 2004.02.21 */

/* smmothing size for peak detection.
 * added at ver.0.2.6
 */
#define SMOOTH_SIZE 3
#define CG_RECT 10

/* default diff clipping factor */
#define CLIP 3.0


/* for cap_window.c */ 
#define DEF_TIMER1 3000  /* default time interval of dv read-out */
#define DEF_TIMER2 3000 /* default time interval of dv read-out in analysis mode, v0.4.0u 2003.05.17 */
#define DEF_TIMER3 60000 /* time interval for command check v0.6.2 2004.11.28 */
#define NSEQ        50 /* number of dv frames for analysis, v0.4.0u 2003.05.16 */
#define NAVERAGE    50 /* number of data points to obtain average peak value v0.6.2 20041127*/
#define LOW_PEAK    40 /* lower limit of averaged peak v0.6.2 */
#define HIGH_PEAK  100 /* upper limit of averaged peak v0.6.2 */
#define OPT_PEAK    75 /* optimum averaged peak v0.6.2 */

/* parameters depending on hardware
 * you should change these for new hardware.
 * 2003.05.xx for Meade DIMM with DV
 */
#define DIAM 0.05     /* diameter of apertures in unit of metre */
#define SEP  0.144    /* separation of apertures in unit of metre */
#define PIXSCALEX 0.482 /* pixel scale in unit of arcsec/pix 2003.05.09*/
#define PIXSCALEY 0.535 /* pixel scale in unit of arcsec/pix 2003.05.09*/
#define LAMBDA 0.55e-6 /* observing wavelength in unit of metre */
#define MIN_HBOX_TIME 1.0 /* minimal time interval for sending handbox command v0.4.8 20030909 */
#define MAX_HBOX_TIME 5.0 /* maximal time interval for sending handbox command v0.5.0 20030910 */
#define SHUTTER_INTV 6666 /* default I.I.CCD shutter interval v0.5.12 2004.11.18 */

#define IMAGE_DIST 63.0   /* average image distance in I.I. 2003.09.12 */
// #define IMDIST_MAX 80     /* max. acceptable image distance */
/* v.0.5.10 modified. iwata */
#define IMDIST_MAX 150.0     /* max. acceptable image distance */
#define IMDIST_MIN 30.0     /* min. acceptable image distance */

/* humidity limit for operation
 * added v.0.5.5 20040219 
 * v0.7.2 value changed 95 to 85 20050420 */
#define HUM_LIMIT 85

/* factor for extension of telescope motion for high altitude 
 * added v.0.5.10 20040802 iwata */
/* changed from 2 --> 3 v0.6.0 20041123 iwata */
#define HIGH_EXTRA 3
/* threshold altitude for extension */
#define ALT_EXTRA  75

/* number of pointing/guiding trials */
#define MAX_TRY 2
#define MAX_TRY_POINTING 10
#define MAX_TRY_GUIDE 20

/* duration of long sleep in minutes 
 * added v0.7.2 20050318 */
#define LONG_SLEEP_MIN 60

/* definition of functions */
/* cap_window.c */
int init_cap_window(GtkWidget *window_cap);
void callback_open_window_cap (GtkWidget *widget, gpointer data);
void open_window_cap (void); /* added v0.6.0 20041122 iwata */
void timeout_reload_image(gpointer pointer);
void reset_peak_history(void); /* added v0.6.2 20041127 iwata */
int get_peak_average(void);    /* added v0.6.2 20041127 iwata */
void starttimer(void);
void stoptimer(void);

/* cap_window_callbacks.c */
gboolean on_darea_expose (GtkWidget *widget, GdkEventExpose *event, gpointer user_data); /* callback for expose event */
int on_click_darea (GtkWidget *widget, GdkEventButton *event);    /* callback for a mouse click */
gboolean on_lamp_expose (GtkWidget *widget, GdkEventExpose *event);    /* callback for expose event */
void callback_track_button(GtkWidget *widget, gpointer pointer);  /* callback for track star button */
void callback_ana_button(GtkWidget *widget, gpointer pointer);    /* callback for analysis button; v0.4.0u, 2003.05.16 */
void callback_record_button(GtkWidget *widget, gpointer pointer); /* callback for record button */
void callback_reset_starpos(GtkWidget *widget, gpointer pointer); /* callback for reset star pos. button */
void callback_reload_image(GtkWidget *widget, gpointer pointer);  /* reload button */
void reset_starfinding(void);                                     /* reset starfinding */
void callback_reset(GtkWidget *widget, gpointer pointer);         /* reset button */
void callback_set_targetpos(GtkWidget *widget, gpointer pointer); /* Set Currnet Finder Pos. as Target v.0.4.8b3 20030909 */
void callback_point_and_center(GtkWidget *widget, gpointer data); /* v0.5.1 */
void timeout_reload_image(gpointer pointer); /* callback for timeout reload */
void callback_stopop_button(GtkWidget *widget, gpointer pointer);  /* callback for stop-op button */
void dialog_stopop_ok(GtkWidget *widget, gpointer data);
void callback_open_window_cap (GtkWidget *widget, gpointer data); /* callback for opening capture window */
void callback_hide_window_cap (GtkWidget *widget, gpointer data); /* callback for hiding capture window */

/* in get_cg.c */
int alloc_rect(int n, int tx, int ty); /* allocate memory for pImRect */
int calc_cg(int n); /* calc center of gravity */
int calc_r0(void);  /* calc r0 and output to logfile */
int calc_r0_new(void);  /* calc r0 and output to logfile v.0.5.1 */
int check_cg(void); /* verify obtained cg */
int ana_r0(void);   /* analysis ver. of calc_r0, v0.4.0u, 2003.05.17 */
void copy_diffcg(int n); /* added v0.6.4 20041202 */
int cmp_diffcg(int n);   /* added v0.6.4 20041202 */


/* in guid.c */
void check_guid(void); /* check guinding is needed */
void start_guid(void); /* exec guiding */

/* in find_star.c */
int get_stars(void); /* find stars in image */
int get_finder_starpos(void); /* get star position in finder. v0.4.8 2003.09.09 */
int is_cloudy(void); /* judge if it is cloudy from peak count of finder */

/* obj_dialog.c */
int init_obj_dialog(void);

/* callback.c */
void callback( GtkWidget *widget, gpointer data );
gint delete_event( GtkWidget *widget, GdkEvent *event, gpointer data );
void callback_open_window (GtkWidget *widget, gpointer data);
void callback_hide_window (GtkWidget *widget, gpointer data);
void callback_close_window(GtkWidget *widget, gpointer data);
void callback_open_setting(GtkWidget *widget);
void mydialog(char *str); /* generic info dialog. v.0.5.1- */
void user_quit(void);

/* obj.c */
int read_edb(char* fname);
GtkListStore* setup_objlist(int num);
int update_objazel(int num, GtkListStore *store);
void update_object_list(void);
int reset_obj_flags(void); /* v0.6.2 */

/* menu.c */
GtkWidget* init_main_menu(GtkWidget *window);

/* for time.c */
typedef struct {
  int year;
  int month;
  int day;
  int hour;
  int min;
  int sec;
} myTime;
/* in time.c */
int get_time(myTime *ttime);  /* get current time */
int get_localtime(myTime *ttime); /* get local time */
double myCalcmjd(int yy, int mm, int dd, int hr, int mn);
double get_lst(double tmjd, double lon);
int get_azel(int onum, double lst);
int get_alt(double ra, double dec, double *alt, double *ha); /* get el for a ra-dec pair */

void cal_mjd (int mn, double dy, int yr, double* tmjd);
void mjd_cal (double tmjd, int* mn, double* dy, int* yr);
void mjd_year (double tmjd, double* yr);
double mjd_day(double jd);

/* used in list model for object data */
enum {
  OBJ_NAME, 
  OBJ_RA, 
  OBJ_DEC, 
  OBJ_AZ,
  OBJ_EL,
  OBJ_MAG,
  N_COLUMNS};



/* conversions among hours (of ra), degrees and radians. */
#define	degrad(x)	((x)*PI/180.)
#define	raddeg(x)	((x)*180./PI)
#define	hrdeg(x)	((x)*15.)
#define	deghr(x)	((x)/15.)
#define	hrrad(x)	degrad(hrdeg(x))
#define	radhr(x)	deghr(raddeg(x))


/* from libastro/preferences.h */
typedef enum {
    PREF_EQUATORIAL, PREF_UNITS, PREF_DATE_FORMAT, PREF_ZONE, PREF_DPYPREC,
    PREF_MSG_BELL, PREF_PRE_FILL, PREF_TIPS, PREF_CONFIRM, PREF_WEEKSTART,
    NPREFS
} Preferences;

typedef enum {PREF_MDY, PREF_YMD, PREF_DMY} PrefDateFormat;

/* in misc.c */
void set_ppm_path();

char* syserrstr ();
char* getShareDir();
char* getPrivateDir();

void db_write_line (Obj * op, char * lp);

/* in telctl_window.c */
int init_telctl_window(GtkWidget *window_telctl);

/* in telctl_callbacks.c */
void update_tel_stat_entry(void);
void callback_connect_telescope(GtkWidget *widget, gpointer data);
void callback_point_telescope(GtkWidget *widget, gpointer data);
void callback_point_telescope2(void); /* ver. 0.4.8 */
void callback_sleep_telescope(GtkWidget *widget, gpointer data); /* ver. 0.4.6 */
/* void callback_stop_telescope(GtkWidget *widget, gpointer data); */
void callback_tel_ctl(GtkWidget *widget, gpointer data);
void callback_tel_ctlstop(GtkWidget *widget, gpointer data);
void callback_tel_setstep1(GtkWidget *widget, gpointer c_toggle);
void callback_tel_setstep2(GtkWidget *widget, gpointer c_toggle);
void callback_tel_setstep3(GtkWidget *widget, gpointer c_toggle);
void callback_tel_setstep4(GtkWidget *widget, gpointer c_toggle);
void callback_focus_pos(GtkWidget *widget, gpointer data); /* v0.5.0 */
void callback_focus_neg(GtkWidget *widget, gpointer data); /* v0.5.0 */
void callback_focusstop(GtkWidget *widget, gpointer data); /* v0.5.0 */


/* for test in telctl_callbacks.c */
void build_msg(void);

/* in telctl_automatic.c */
int pointing_proc(void);       /* pointing procedure */
int autoselect_star(void);     /* automatically select a star for DIMM measurement */
int autoselect_fainter_star(float dmag); /* automatically select a star, fainter than currently selected star v0.5.7 2004.02.21 */
int point_meade(void);         /* point telescope to selected star */
int point_finder_target(void); /* move star to target position in finder image v0.5.0 2003.09.10 */
int wait_finder_fixed(void);   /* wait for star image in finder fixded */
int recover_stars(void);       /* procedure for recovering stars in I.I. image v.0.5.3 2003.10.03 */
int check_auto_status(void);   /* check status for automatic mode */
void stop_operation(void);     /* automatically stop all operations v0.5.4 */
int check_end_time(void);      /* end time check v0.5.6 */
int check_alt(void);           /* altitude check v0.5.6 */
int check_environ(void);       /* rain and humidity check v0.5.6 */
int set_iishutter(int t);      /* set IICCD shutter interval added v0.5.12 2004.11.18 */
int long_sleep(int min);       /* v0.6.5 20041208 */
void resume_obs(void);         /* v0.6.5 20041208 */

/* in autopoint_procedure.c */
int pointproc01(void);
int pointproc02(void);
int pointproc03(void);
int pointproc04(void);
int pointproc05(void);
int pointproc06(void);

/* in telescope2.c */
int linkTel(void); /* void --> int v0.6.0 2004.11.22 iwata */
void unlinkTel(void);
void showTelRA();          /* display latest telescope RA  */
void showTelDec();         /* display latest telescope Dec */
void set_target_by_object(void);
void slewTel();


/* in prefs.c */
int init_setup_dialog(GtkWidget *window_setup);
int get_finder_target_pos(void); /* read finder target position from a file added on v0.5.6 2004.02.20 */
int read_config(char* fname);    /* read from a configuration file v0.6.4 20041203 iwata */

/* outputlog.c */
int setoutputfile (void);
void closeoutputfile(void);

/* in opmode.c */
void toggle_opbutton(GtkWidget *widget, gpointer data);
void toggle_guidebutton(GtkWidget *widget, gpointer data);
int init_starsel_dialog(void); /* auto-select star setting dialog */
void update_main_displays(void); /* update displays in Main Window */
void dialog_click_cancel (GtkWidget *widget, gpointer data);
void dialog_click_ok (GtkWidget *widget, gpointer data);

/* in socket_client.c */
int connect_socket(char *hostname);
int read_finder_coords(void);
int disconnect_socket(void);

/* in command.c */
void start_comm_timer(void);
void stop_comm_timer(void);
void timeout_comm_check(gpointer pointer);


/* observation site info */
typedef struct {
  double slon;
  double slat;
  unsigned char lt;
  char name[256];
} Site;

/* pixel coords */
typedef struct {
  int cx;
  int cy;
} pixcoord;

/* coord in double */
typedef struct {
  double x;
  double y;
} dcoord;

/* seemon2 global settings */
typedef struct {
  int objnum;      /* number of objects in object list */
  int selnum;      /* selected object number */
  int time_reload; /* interval for reload dv in msec */
  int time_peak;   /* interval for peak check v0.5.7 2004.02.21 */
  int cnt_peak;    /* counter for peak check v0.5.7 2004.21.21 */
  int time_redraw; /* interval for drawing, added v0.3.0 2003.05.16 */
  int time_reload1; /* reload interval for no-tracking mode */
  int time_reload2; /* reload interval for tracking mode */
  int time_iishutter;  /* current IICCD shutter interval. v0.5.12 2004.11.18 */
  int time_iishutter0; /* default IICCD shutter interval. v0.7.2 20050209 */
  int time_reload_comm;  /* command-chekc interval. v0.6.2 2004.11.26 */
  //  int skyval;      /* background sky value */ // hidden v0.5.7 2004.02.21 
  int peakval[2];  /* stellar peak values */
  int peakhistory[NAVERAGE]; /* for averaged peak value. v0.6.2 20041127 */
  int peak_average; /* v0.6.2 20041127 */
  int starflux[2]; /* sumintensity in calc_cg(). v0.5.11 2004.11.6 Hattori */
  int min_peak;    /* minimum peak value */
  char flg_reload; /* flg for reloading */
  char msg[256];   /* for goto command to telescope */
  double tel_ra;   /* current ra in radian */
  double tel_dc;   /* current dc in radian */
  double tel_el;   /* current elevation in radian */
  double tel_ha;   /* current hour angle */
  int tel_step;    /* motion step in arcsec */
  double trg_ra;   /* target ra  in radian */
  double trg_dc;   /* target dec in radian */
  char dvfilepath[256];   /* path of .ppm file (converted from dv image) */
  char dvseqpath[NSEQ][256]; /* path of sequential .ppm files */
  int tel_stat;    /* telescope status. 0) disconnect 1) connected 2) pointing 3) pointed 4) sleeping */
  pixcoord star_center[2];
  pixcoord star_rect[2];
  pixcoord cg_rect[2]; /* rect. size for cg calc. added ver.0.2.6 */
  pixcoord peak[2];    /* star peaks, added ver. 0.2.6 */
  pixcoord prevgdiff[2];  /* previous dx,dy v0.6.8 20050110 */
  pixcoord finder_star_center; /* star position in finder. added ver.0.4.8 2003.09.09 */
  pixcoord finder_targetpos;   /* target position in finder. added ver.0.4.8 2003.09.09 */
  dcoord cg[2];
  dcoord oldcg[2];
  dcoord diffcg[N_CALC_R0];
  dcoord prev_diffcg[N_CALC_R0]; /* added v0.6.4 20041202 iwata */
  dcoord r0;
  dcoord fwhm;
  dcoord fwhm_corr; /* corrected FWHM */
  double clip;      /* sigma clipping factor, added ver. 0.2.6 */
  int opmode;       /* operation mode: 0:manual 1:automatic, added ver. 0.4.6 */
  int op_ssmode;    /* automatic star selection mode 0:auto 1:point_current 2:already_pointed added v0.5.4 */
  int is_capturing; /* analyzing=2, capturing=1, otherwise 0 */
  int is_tracking;  /* tracking=1; not tracking=0 */
  int is_recording; /* recording=1 */
  int is_guiding;   /* guinding=1. v.0.4.1 2003.05.28 */
  int is_pointing;  /* pointing > 1; see telctl_automatic.c v0.5.0 2003.09.10 */
  int is_sleeping;  /* long sleep v0.6.8 (also counter) */
  int sleep_length; /* long sleep duration in min */
  int proc_cnt;     /* counter for pointproc0? v0.5.6 2004.02.19 */
  int motionflg;    /* checker for large cg motion. 0 if motion is large */
  int guideflg;     /* guide function on/off v.0.4.7 */
  myTime ctime;     /* (current) time */
  int tel_dir[4];   /* info of telescope handbox buttons */
  double image_dist; /* distance of two stellar images (in pixels)
		      * used for tracking check */
  float bound;      /* fraction of fov needed for guinding v0.4.1 2003.05.28 */
  int finder_peakval;       /* peak count in finder image */
  int finder_dist_threshold; /* for point_finder_target() v0.5.3 2003.10.03 */
  int finder_dist;          /* star distance in finder v0.7.2 2005.02.11 */
  int finder_dist_prev;     /* previous distance in finder v0.7.2 2005.02.11 */
  double image_theta[NSEQ]; /* axis angle of images (each frame) */
  double image_theta0;      /* axis angle of images (average; used as standard) */
  //  int thetaflg;             /* axis angle is set? (0: false 1:true) */
  //  dcoord axis;      /* normalized axis vector */
  double endtime;   /* end time of operation in automatic mode (localtime) */
  int rainflg;      /* 1:rain 0:no rain added v0.5.5 2004.02.19 */
  int humidity;     /* added v0.5.5 2004.02.19 */
  int domestatus;   /* added v0.5.5 2004.02.19 */
  int high_extra;   /* acceleration factor for high altitude. added v0.5.10 20040802 iwata */
  int alt_extra;    /* lower limit of acceleration altitude added v0.5.10 20040802 iwata */
  int autoflg;      /* 1:auto 0:manual v0.6.0 2004.11.22 */
  int dimm_id;      /* dimm telescope id, added v.0.5.11 20041102 iwata */
  int ntry1;        /* number of pointing trial v0.6.2 2004.11.27 */
  int ntry2;        /* number of guiding trial v0.6.2 2004.11.27 */
  int n_average;    /* number of average peak data v0.6.2 2004.11.27 */
  float alt_limit1; /* lower limit of star altitude for selection v0.6.5 20041208 */
  float alt_limit2; /* lower limit of star tracking v0.6.5 20041208 */
} Seemon;


/******* 
 * global variables
 *******/
#define MAXOBJNUM 999
/* object-related variables */
Obj* obj_edb[MAXOBJNUM];
char* obj_line[MAXOBJNUM];
GtkListStore* objlist;
char edbpath[256]; /* path of edb file */
char tgtpath[256]; /* path of finder target position file. added v0.5.6 2004.02.20 */
/* int redraw_count; /* redraw counter */

guchar* graybuf; /* pointer for gray-scale image buffer (read from DV) */
int wid, hgt;    /* image size */
char flg_set_rect[2]; /* flags to judge rectangles are set */

/* variables used in get_cg.c */
int *pImRect[2]; /* pointers for rectangular image buffer */
double bgthr; /* background subtraction threshold */
double cgfactor; /* cg adjust factor, imported from seemon1 */
int n_calc_r0;	 /* number of data points for calculating r0 */
//int n_totalsample; /* number of total data points obtained */ /* commented out v0.5.10 20040802 iwata */
double scalex, scaley;
double Lambda;   /* observing wavelength in metre*/
dcoord Sigma, Mean, r0k;
int NSample, Ndata;

/* boundaries for guinding 
 * v.0.4.1 2003.05.28
 */
int boundx1, boundx2, boundy1, boundy2;

/* output file pointer */
FILE *fout;

/* socket */
//#define SOCKET_SERVER "dimm2"
int fd_socket; /* v.0.4.8b2 */
char socket_server[30];

/* site info */
Site msite;

/* generic global settings */
Seemon mseemon;

/* widgets */
GtkWidget* ent_selobj1;
GtkWidget* ent_tel_stat1;
GtkWidget* ent_opmode;            /* operation mode, added ver.0.4.6 */
GtkWidget* ent_guidemode;         /* guide mode, added ver.0.4.7 */
GtkWidget *ent_tel_ra, *ent_tel_dc;
GtkWidget *ent_trg_ra, *ent_trg_dc;
GtkWidget *but_tel_conn, *but_tel_sleep;
GtkWidget *ent1, *ent2, *ent3, *ent4, *ent5, *ent6, *ent7;  /* used in prefs.c */
GtkWidget *ent_intv, *ent_intv2, *ent_intv3;          /* used in prefs.c */
GtkWidget *ent_wid, *ent_hgt, *ent_bgsig; /* used in prefs.c */
GtkWidget *ent_cgwid, *ent_cghgt; /* used in prefs.c, added ver.0.2.6 */
GtkWidget *ent_clip;              /* used in prefs.c, added ver.0.2.6 */
//GtkWidget *ent_redraw;          /* used in prefs.c, added ver.0.3.0 */
//GtkWidget *ent_skyintv;         /* used in prefs.c */
GtkWidget *ent_minpeak;           /* used in prefs.c, added ver.0.5.1 */
GtkWidget *ent_fndr_targetx;      /* used in prefs.c, added ver.0.5.3 */
GtkWidget *ent_fndr_targety;      /* used in prefs.c, added ver.0.5.3 */
GtkWidget *ent_ext1, *ent_ext2;   /* used in prefs.c, added ver.0.5.10 20040802 iwata */
GtkWidget *ent_h, *ent_m;         /* used in telctl_automatic.c v0.5.3 */
GtkWidget *g_image_area1;         /* captured image area, used in cap_window.c */
GtkWidget *g_lamp_area1, *g_lamp_area2, *g_lamp_area3;  /* used in cap_window.c; v0.4.0u */
GtkWidget *btn1, *btn2, *btn3;    /* used in cap_window.c and cap_window_callbacks.c */
GdkColor  mcolor[4];  /* colors v0.4.0u */
GtkWidget *ent_r0;        /* current r0 in cap_window.c */
GtkWidget *toggle_cstep; /* current selected toggle button for tel. step. used in telctl_window.c */
GtkWidget *window_capture;     /* capture window, v.0.4.8 */
gint timertag;       /* timer tag, used in cap_window.c */
gint timertag_comm;  /* timer tag for command, used in command.c */
