/*
 * for automatic telescope control
 *
 * 2003.08.16 -
 * Time-stamp: <2005-03-20 12:53:48 dimm>
 *
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <gtk/gtk.h>

#include "seemon2.h"
#include "protocol.h"

/*************************
 * mseemon.is_pointing
 * 0: not in pointing mode
 * 1: select new star
 * 2: point meade
 * 3: wait for star image in finder fixed
 * 4: move star to target position in finder image
 * 5: check star is in I.I. image
 * 6: centering in I.I. image
 *************************/

/*
 * automatic pointing procedure
 *
 * v0.5.0 2003.09.10
 * v0.5.1 2003.10.02 added call of dialog asking end time
 */
int pointing_proc(void) 
{
  char str[20];
  int ret;

  if (fout != NULL) {
    /* update ctime */
    get_localtime(&(mseemon.ctime));

    sprintf(str, "%04d.%02d.%02d-%02d:%02d:%02d", 
	  mseemon.ctime.year, 
	  mseemon.ctime.month,
	  mseemon.ctime.day,
	  mseemon.ctime.hour,
	  mseemon.ctime.min,
	  mseemon.ctime.sec);

    fprintf(fout, "### %s pointing_proc is_pointing=%d ntry1=%d\n", 
	    str, 
	    mseemon.is_pointing, mseemon.ntry1);
  }
  #ifdef DEBUG
  printf("### %s pointing_proc is_pointing=%d ntry1=%d\n", str, 
	 mseemon.is_pointing, mseemon.ntry1);
  #endif

  switch(mseemon.is_pointing) {
  case 0:
    ret = EXIT_FAILURE; /* error */
    break;
  case 1:
    ret = pointproc01(); /* select star */
    break;
  case 2:
    ret = pointproc02(); /* point meade */
    break;
  case 3:
    ret = pointproc03(); /* wait star position in finder fixed */
    break;
  case 4:
    ret = pointproc04(); /* move star to target position in finder image */
    break;
  case 5:
    ret = pointproc05(); /* check star is in I.I. image */
    break;
  case 6:
    ret = pointproc06(); /* 6: centering in I.I. image */
    break;
  default:
    printf("# invalid value of is_pointing: %d\n", mseemon.is_pointing);
    if (fout != NULL) {
      fprintf(fout, "# invalid value of is_pointing: %d\n", mseemon.is_pointing);
    }
    ret = EXIT_FAILURE; /* error */
  }

  /* if failed more than MAX_TRY times, flag the selected star */
  if (ret == EXIT_FAILURE) {
    mseemon.ntry1++;
    if (mseemon.ntry1 > MAX_TRY) {
      obj_edb[mseemon.selnum]->o_flags = 1;
      mseemon.ntry1 = 0;
    }
  }

  /* update Main Window text entries */
  update_main_displays();

  /* write to logfile */
  if (fout != NULL) fflush(fout);

  return(ret);
}


/* automatically select a star for DIMM measurement 
 * 
 * v.0.5.0 2003.09.10
 * update v0.6.2 20041127 check obj_edb[i]->s_flags
 */
int autoselect_star(void) {
  int i, candstr, flg, flg2;
  myTime mt;
  double d, tmjd, lst, min_ha, ha;
  float alt;
  char str[10];
  gchar text[128];
  gchar* content;

  min_ha = 4.0;
  candstr = -1;

  /* update ctime, LST, AZ, EL */
  get_localtime(&(mseemon.ctime));
  get_time(&mt);
  d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
  tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
  lst = get_lst(tmjd, msite.slon);

  #ifdef DEBUG
  printf("# current LST: %6.3f\n", lst);
  #endif

  flg = 1;
  flg2 = 0;

  while (flg) {
    for (i=0; i < mseemon.objnum; i++) {
      if (obj_edb[i]->o_flags == 0) { /* not flagged*/
	get_azel(i, lst);
	alt = (float)(obj_edb[i]->s_alt * 180. / PI); /* in deg */
	ha = obj_edb[i]->s_ha * 12. / PI; /* in hour */
	if (ha < -8) ha += 24.;
	if (ha > 12) ha -= 24.;
	#ifdef DEBUG
	printf("%2d: ra:%6.3f ha:%5.1f alt:%5.1f\n", i, (obj_edb[i]->s_ra * 12. / PI), ha, alt);
        #endif
	if (ha < min_ha && alt > mseemon.alt_limit1) {
	  min_ha = ha;
	  candstr = i;
	}
      }
    }

    if (candstr >= 0) {
      mseemon.selnum = candstr;
      alt = (float)(obj_edb[candstr]->s_alt * 180. / PI);
      #ifdef DEBUG
      printf("autoselect_star(): \n");
      printf(" candidate: %s HA:%4.1f ALT:%4.1f\n", 
	     obj_edb[candstr]->o_name, min_ha, alt);
      #endif
      if (fout != NULL) {
	fprintf(fout, "# autoselect_star(): ");
	fprintf(fout, "candidate: %s HA:%4.1f ALT:%4.1f\n", 
		obj_edb[candstr]->o_name, min_ha, alt);
      }
      mseemon.selnum = candstr;
      flg = 0; /* found candidate successfully */
    } else {
      if (flg2 == 0) {
	printf("# autoselect_star(): no appropriate star found. \n");
	if (fout != NULL) {
	  fprintf(fout,"# autoselect_star(): no appropriate star found. \n");
	}
	reset_obj_flags(); /* in obj.c */
	flg = 0;
	flg2 = 1;
      } else {
	printf("# autoselect_star() failed: no appropriate star found.\n");
	if (fout != NULL) {
	  fprintf(fout, "# autoselect_star() failed: no appropriate star found.\n");
	}
	return(EXIT_FAILURE);
      }
    }
  }


  if (flg2) long_sleep(LONG_SLEEP_MIN); /* sleeping 30 minutes */


  /* update displays */
  strcpy (text, obj_edb[mseemon.selnum]->o_name);
  content = text;
  gtk_entry_set_text (GTK_ENTRY (ent_selobj1), content);

  /* set target */
  set_target_by_object(); /* in telescope2.c */
  showTargetRA();
  showTargetDec();

  return(EXIT_SUCCESS);
}


/* move star to target position in finder image
 *
 * returns: 1: error
 *          0: pointing command out
 *          -1: star is at taget position; no more motion needed
 *
 * 2003.09.10
 */
int point_finder_target(void) 
{
  int i, ret, dx, dy, x1, y1, tx, ty;
  int teldir, itime;
  int flgx = 0, flgy = 0;
  double dtime;
  char str[10];
  double d, tmjd, lst, alt, dist;
  myTime mt;

  int threshold; /* if distance is above this, more motion needed */
  threshold = mseemon.finder_dist_threshold;

  if (mseemon.finder_targetpos.cx < -500) {
    printf("# finder taget position is not set!\n");
    return(EXIT_FAILURE);
  }

  ret = get_finder_starpos(); /* in find_star.c */

  if (ret == EXIT_FAILURE) {
    printf("# error in reading finder position.\n");
    if (fout != NULL) {
      fprintf(fout, "# error in reading finder position.\n");
    }
    return(EXIT_FAILURE); /* failed */
  }

  if (mseemon.finder_peakval < FNDR_MIN_PEAK) {
    printf("# finder peak value %d is too low.\n", 
	   mseemon.finder_peakval);
    return(EXIT_FAILURE); /* failed */
  }

  /* ok. go ahead */
  tx = mseemon.finder_targetpos.cx;
  ty = mseemon.finder_targetpos.cy;
  x1 = mseemon.finder_star_center.cx;
  y1 = mseemon.finder_star_center.cy;
  
  dx = x1 - tx;
  dy = y1 - ty;
  mseemon.finder_dist = (int)sqrt(dx*dx+dy*dy);

  #ifdef DEBUG
  printf("target:x:%d,y:%d dx:%3d dy:%3d dist:%3d\n", tx, ty, dx, dy, mseemon.finder_dist);
  #endif
  if (fout != NULL) {
    fprintf(fout, "# point_finder_target(): target:x:%d,y:%d dx:%3d dy:%3d dist:%3d\n", tx, ty, dx, dy, mseemon.finder_dist);
  }

  /* set slew rate to CENTER */
  /* added by iwata v0.5.10 20040802 */
  SetRateCenter0();
  mseemon.tel_step = CENTER; /* added by iwata v0.5.10 20040810 */

  /* pointing for x direction */
  if (dx > threshold || dx < -1 * threshold) {
    flgx = 1;
    /* change dir for different DIMM
     * 20041101 v0.5.11 iwata */
    switch (mseemon.dimm_id) {
    case 1:
      if (dx > 0) {
	teldir = WEST;
      } else {
	teldir = EAST;
	dx *= -1;
      }
      break;
    default:
      if (dx > 0) {
	teldir = EAST;
      } else {
	teldir = WEST;
	dx *= -1;
      }
    }
//    dtime = dx * 1000000 / (16 * 15.0417);
    // changed by T.Hattori 2004/07/22
    get_localtime(&(mseemon.ctime));
    get_time(&mt);
    d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
    tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
    lst = get_lst(tmjd, msite.slon);
    get_azel(mseemon.selnum, lst);
    alt = cos(obj_edb[mseemon.selnum]->s_alt);
    if (alt < 0.05) alt = 0.05;

    /* time in usec */
    /* CENTER is 16xsiderial motion */
    dtime = dx * 1000000 / (alt * 16 *15.0417);
    #ifdef DEBUG
    printf("cos(alt)=%f, dtime=%f\n",alt,dtime/1000000);
    #endif
    // end of change

    /* addition by iwata v0.5.10 20040802 */
    alt = obj_edb[mseemon.selnum]->s_alt * 180. / PI;
    if (alt > mseemon.alt_extra) {
      dtime *= mseemon.high_extra;
      #ifdef DEBUG
      printf("alt=%5.2f. Extend motion time by factor of %d: dtime=%6.3f\n", obj_edb[mseemon.selnum]->s_alt, mseemon.high_extra, dtime/1000000);
      #endif
    } /* end of addition */

    itime = (int)dtime;

    sleep(1);

    StartSlew(teldir);
    usleep(itime);
    StopSlew(teldir);
    //    }
  }

  /* pointing for y direction */
  if (dy > threshold || dy < -1 * threshold) {
    flgy = 1;
    if (dy > 0) {
      teldir = SOUTH;
    } else {
      teldir = NORTH;
      dy *= -1;
    }
    dtime = dy * 1000000 / (16 * 15.0417);
    itime = (int)dtime;

    sleep(1);
      
    StartSlew(teldir);
    usleep(itime);
    StopSlew(teldir);
  }

  /* check if star is in I.I. image */
  dvgrab();
  load_pix_buffer();
  copy_to_gray_buffer();
  redraw_image();

  get_stars();
  for (i=0; i<2; i++) {
    search_peak(i);
    calc_cg(i);
  }

  #ifdef DEBUG
  printf("# star peaks: %d %d limit:%d\n", mseemon.peakval[0], mseemon.peakval[1], mseemon.min_peak);
  #endif

  /* check peak value and image distance 
   * to confirm star is in I.I. FOV */
  if (mseemon.peakval[0] > mseemon.min_peak &&
      mseemon.peakval[1] > mseemon.min_peak) {
    if ((ret = check_cg())==0) {
      /* no more motion is needed */
      mseemon.finder_dist_threshold = 10;
      return(-1);
    }
  }

  /* star is not in I.I. 
   * make dist. threshold small 
   * and try again
   */
  if (flgx == 0 && flgy == 0) {
    mseemon.finder_dist_threshold = 5;
    #ifdef DEBUG
    printf("# finder pos. threshold is now %d.\n", threshold);
    printf("# sleeping for a while...\n");
    #endif
    sleep(5);
  } else {
    /* wait 5 sec. for updating finder pos. */
    #ifdef DEBUG
    printf("sleeping for a while...\n");
    #endif
    sleep(5);
  }

  return(0);
}


/* wait for star image in finder fixed */
int wait_finder_fixed(void){
  int ret, cnt, flg = 1;
  int peak;
  pixcoord p1, p2, diff;
  
  int threshold, peakthreshold;

  cnt = 0;
  threshold = 2;      /* position diff. */
  peakthreshold = FNDR_MIN_PEAK; /* minimum peal value in finder image */

  printf("# wait for star image in finder fixed...\n");
  p1.cx = -100;
  p2.cy = -100;

  while (flg) {
    ret = get_finder_starpos(); /* in find_star.c */
    if (ret == EXIT_SUCCESS) {
      p2.cx = mseemon.finder_targetpos.cx;
      p2.cy = mseemon.finder_targetpos.cy;
      peak = mseemon.finder_peakval;

      diff.cx = p2.cx - p1.cx;
      diff.cy = p2.cy - p1.cy;

      if (diff.cx < threshold &&
	  diff.cx > -1 * threshold && 
	  peak > peakthreshold) {
	flg = 0;
      } else {
	cnt++;
	if (cnt > MAX_TRY_POINTING) {
	  printf("# timeout while waiting for finder image fixed.\n");
	  if (fout != NULL) {
	    fprintf(fout, "# timeout while waiting for finder image fixed.\n");
	  }
	  return(EXIT_FAILURE); /* failed */
	}
	if (peak > peakthreshold) {
	  p1.cx = p2.cx;
	  p1.cy = p2.cy;
	}
	/* wait 5 sec. */
	sleep(5);
      }
    } else {
      printf("# error in reading finder position.\n");
      if (fout != NULL) {
	fprintf(fout, "# error in reading finder position.\n");
      }
      return(EXIT_FAILURE); /* failed */
    }
  }

  return(EXIT_SUCCESS); /* succeeded */
}


/* procedure for recovering stars in I.I. image 
 *  using finder image.
 *
 * NOTE: it is assumed that star is in finder image.
 *
 * returns: 1: error
 *          0: success
 */
int recover_stars(void) {
  int ret, flg, cnt, ecnt, success;

  /* check if cloudy */
  if ((ret = is_cloudy())==-1) {
    printf("it seems cloudy.\n");
    if (fout != NULL) {
      fprintf(fout, "# recover_stars: it seems cloudy.\n");
    }
    return(EXIT_FAILURE);
  }

  /* check whether finder target pos. is set */
  if (mseemon.finder_targetpos.cx < -500) {
    printf("*** finder target posistion is not set! ***\n");
    return(EXIT_FAILURE);
  }

  cnt = 0;
  ecnt = 0;
  flg = 1;
  success = 0;

  while (flg) {
    ret = point_finder_target();
    #ifdef DEBUG
    printf("# point_finder_target result:%d.\n",ret);
    #endif

    if (ret == -1) {
      /* succeeded in reading position in finder*/
      #ifdef DEBUG
      printf("# succeeded in reading position in finder.\n");
      #endif
      flg = 0;
      success = 1;
    } else if (ret == 1) {
      ecnt++;
      if (ecnt >= 5) {
	/* error */
	printf("error in point_finder_target().\n");
	return(EXIT_FAILURE);
      }
    } else {
      /* need more motion */
      cnt++;
      if (cnt >= 40) {
	/* timeout */
	printf("timeout while recovering star.\n");
	if (fout != NULL) {
	  fprintf(fout, "# timeout while recovering star.\n");
	}
	return(EXIT_FAILURE);
      }
    }
  }

  /* success */
  return(EXIT_SUCCESS);

}

/* point telescope to selected star */
int point_meade(void) {
  int ret, i;

  /* pointing */
  if (mseemon.tel_stat > 0) {
    fprintf(fout, "# autopoint: try pointing. object number=%d\n", mseemon.selnum);
    printf("try pointing. object number=%d\n", mseemon.selnum);
    callback_point_telescope2(); /* in telctl_callbacks.c */
  } else {
    printf("telescope is not connected or object is not selected; abort autopoint.\n");
    fprintf(fout, "# autopoint: telescope is not connected or object is not selected; abort autopoint.\n");
    return(EXIT_FAILURE);
  }

  return(EXIT_SUCCESS);
}


/* check status for automatic mode 
 * v0.5.0 2003.09.10
 ***** NEED to be fixed(while pointing_proc) ****
 */
int check_auto_status(void) {
  int ret;

  /* check telescope status */
  if (mseemon.is_pointing == 0) {     /* normal operation */
    if (mseemon.tel_stat < 2) {
      /* telescope is not pointed; exec pointing */
      printf("# check_auto_status: telescope not pointed: stat=%d\n", mseemon.tel_stat);
      printf("# exec pointing\n");
      if (fout != NULL) {
	fprintf(fout, "# check_auto_status: telescope not pointed: stat=%d\n", mseemon.tel_stat);
	fprintf(fout, "# exec pointing\n");
      }
      mseemon.is_pointing = 1;
      return(EXIT_SUCCESS);
    } else {
      /* end time check */
      //      #ifdef DEBUG
      //      printf("# check_end_time() called from check_auto_status():01.\n");
      //      if (fout != NULL) {
      //	fprintf(fout, "# check_end_time() called from check_auto_status().:01\n");
      //      }
      //      #endif
      ret = check_end_time();

      /* check altitude */
      ret = check_alt();

      /* check cloudy added v0.5.3 */
      ret = is_cloudy();
      if (ret != -1) { /* not cloudy */
	/* check stars are found */
	if (flg_set_rect[0] != 1 || flg_set_rect[1] != 1) {
	  /* find stars and set rects */
	  dvgrab();
	  load_pix_buffer();
	  copy_to_gray_buffer();
	  redraw_image();
	  ret = get_stars();
	    
	  if (ret == EXIT_FAILURE) {
	    printf("# check_auto_status: calling recover_stars()\n");
	    if (fout != NULL) {
	      fprintf(fout, "# check_auto_status: calling recover_stars()\n");
	    }
	    ret = recover_stars();
	    if (ret == EXIT_FAILURE) {
	      printf("# check_auto_status: recover_stars() failed. retry pointing.\n");
	      if (fout != NULL) {
		fprintf(fout, "# check_auto_status: recover_stars() failed. retry pointing\n");
	      }
	      mseemon.is_pointing = 2;
	      #ifdef DEBUG
	      printf("### is_pointing ==> 2 telctl_automatic.c 603\n");
	      #endif
	      return(EXIT_FAILURE);
	    }
	  }
	}
      
	/* modified v0.5.5 20040219 */
	if (mseemon.is_recording != 1) {
	  /* set tracking */
	  mseemon.is_tracking = 1;
	  gtk_button_set_label(GTK_BUTTON(btn1), " Stop Tracking");
	  set_lamp(g_lamp_area1, 1);
      
	  /* set analysis */
	  mseemon.is_capturing = 2;
	  gtk_button_set_label(GTK_BUTTON(btn2), " Stop Analysis");
	  set_lamp(g_lamp_area2, 1);
	  /* set timer time */
	  mseemon.time_reload = mseemon.time_reload2;
	  /* restart timer */
	  if (timertag != -1) {
	    /* stop current working timer */
	    stoptimer();
	  }
	  starttimer();

	  /* set recording */
	  mseemon.is_recording = 1;
	  gtk_button_set_label(GTK_BUTTON(btn3), " Stop Recording");
	  set_lamp(g_lamp_area3, 1);
	}
      } else {
	printf("# it is cloudy :-(\n");
	if (fout != NULL) {
	  fprintf(fout, "# it is cloudy :-(\n");
	}
	/* reset rects, turn off tracking, analysis, recording */
	reset_starfinding(); /* in cap_window_callbacks.c */
      }

      /* check rain and humidity
       * added v.0.5.5 20040219 */
      ret = check_environ();

      return(ret);
    }
  } else {
    /* pointing_proc() running.
     * check alt, end time, rain+humidity */
    ret = check_alt();          /* in telctl_automatic.c */
    //    #ifdef DEBUG
    //    printf("# check_end_time() called from check_auto_status():02.\n");
    //    if (fout != NULL) {
    //      fprintf(fout, "# check_end_time() called from check_auto_status().:02\n");
    //    }
    //    #endif
    ret = check_end_time();     /* in telctl_automatic.c */
    ret = read_finder_coords(); /* in socket_client.c */
    ret = check_environ();
  }
}

/* automatically stop all operations */
void stop_operation(void) {
  char str[20], key[10];

  /* update ctime */
  get_localtime(&(mseemon.ctime));

  sprintf(str, "%04d.%02d.%02d-%02d:%02d:%02d", 
	  mseemon.ctime.year, 
	  mseemon.ctime.month,
	  mseemon.ctime.day,
	  mseemon.ctime.hour,
	  mseemon.ctime.min,
	  mseemon.ctime.sec);

  printf("# %s stop operation. going to sleep..\n", str);
  if (fout != NULL) {
    fprintf(fout, "# %s stop operation. going to sleep..\n", str);
  }

  /* terminate all */
  stoptimer();
  if (mseemon.is_capturing > 0) { /* if capture window is open */
    reset_starfinding(); /* in cap_window_callbacks.c */
    gtk_widget_hide((GtkWidget*)window_capture);
  }
  mseemon.is_pointing = 0;
  mseemon.is_capturing = 0;
  mseemon.is_guiding = 0; /* added v0.5.6 20040220 */
  mseemon.opmode = 0; /* added v0.5.5 20040217 */

  if (mseemon.tel_stat > 0) {
    sleepTel(); /* in protocol.c */
    mseemon.tel_stat = 4; /* sleeping; added v0.5.6 2004.02.20 */
    gtk_button_set_label(GTK_BUTTON(but_tel_sleep), "Wake-up Telescope"); /* update button label; added v0.5.11 2004.11.04 */
  }

  /* send dome close signal */
  if (fd_socket >= 0) {
      printf("# sending finder_comm a dome-close signal.\n");
      if (fout != NULL) {
	fprintf(fout, "# sending finder_comm a dome-close signal.\n");
      }
      strcpy(key, "dclose");
      write(fd_socket, key, sizeof(key));

      sleep(5);
  } else {
    if (fout != NULL) {
      fprintf(fout, "# unable to send finder_comm a dome-close signal, because socket is not connected.\n");
    }
  }

  /* send CCD power-off signal */
  if (fd_socket >= 0) {
      printf("# sending finder_comm a CCD power-off signal.\n");
      if (fout != NULL) {
	fprintf(fout, "# sending finder_comm a CCD power-off signal.\n");
      }
      strcpy(key, "ccdoff");
      write(fd_socket, key, sizeof(key));

      sleep(5);
  } else {
    if (fout != NULL) {
      fprintf(fout, "# unable to send finder_comm a CCD power-off signal, because socket is not connected.\n");
    }
  }

  update_main_displays(); /* in opmode.c */

  if (fout != NULL) {
    closeoutputfile(); /* in outputlog.c */
    fout = NULL;
  }

  /* set timer for start command using scr.checktime02.pl */
  system("/usr/local/bin/scr.checktime02.pl");
  /* send finder_comm to set timer */
  strcpy(key, "timeset");
  write(fd_socket, key, sizeof(key));


  /* wait 30sec for telescope moving toward the rest position */
  printf("# wait 30sec..\n");
  sleep(30);

  unlinkTel();
}

/* end time check. added v0.5.6 2004.02.20 */
int check_end_time(void) {
  int eh, em;
  double t;

  get_localtime(&(mseemon.ctime));
  t = mseemon.ctime.hour + mseemon.ctime.min/60.;
  eh = (int)mseemon.endtime;
  em = (int)((mseemon.endtime-eh)*60.);
  printf("# time to stop op.: %02d:%02d ", eh, em);
  if (t > mseemon.endtime) {
    stop_operation(); /* in telctl_automatic.c */
  }

  return(EXIT_SUCCESS);
}

/* altitude check added v0.5.6 2004.02.20 */
/* modified 20041208 */
int check_alt(void) {
  myTime mt;
  double d, tmjd, lst;
  float alt, min_alt, ha;
  char str[20];

  min_alt = mseemon.alt_limit2; /* if alt below this, go to next */

  get_time(&mt);
  d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
  tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
  lst = get_lst(tmjd, msite.slon);
  get_azel(mseemon.selnum, lst);
  alt = (float)(obj_edb[mseemon.selnum]->s_alt * 180. / PI);
  ha = (float)(obj_edb[mseemon.selnum]->s_ha * 12./PI);

  printf(" current alt:%5.2f HA:%5.2f\n", alt, ha);

  if (alt < min_alt) {
    sprintf(str, "%04d.%02d.%02d-%02d:%02d:%02d", 
	    mseemon.ctime.year, 
	    mseemon.ctime.month,
	    mseemon.ctime.day,
	    mseemon.ctime.hour,
	    mseemon.ctime.min,
	    mseemon.ctime.sec);

    printf("# %s: %s alt. %5.2f. go to next star.\n", 
	   str, obj_edb[mseemon.selnum]->o_name, alt);
    if (fout != NULL) {
      fprintf(fout, "# %s: %s alt. %5.2f. go to next star.\n", 
	      str, obj_edb[mseemon.selnum]->o_name, alt);
    }
    mseemon.is_pointing = 1;
    mseemon.op_ssmode = 0; /* auto select star */
  }

  return(EXIT_SUCCESS);
}

/* rain and humidity check added v0.5.6 2004.02.20 */
int check_environ(void) {
  if (mseemon.rainflg == 1||
      mseemon.humidity >= HUM_LIMIT) {
    printf("# rainflg:%d humidity:%d dome:%d. stop operation.\n",
	   mseemon.rainflg, mseemon.humidity, mseemon.domestatus);
    if (fout != NULL) {
      fprintf(fout, "# rainflg:%d humidity:%d dome:%d. stop operation.\n",
	      mseemon.rainflg, mseemon.humidity, mseemon.domestatus);
    }
    stop_operation(); /* in telctl_automatic.c */
  }
  return(EXIT_SUCCESS);
}


/* set IICCD shutter interval added v0.5.12 2004.11.18 */
int set_iishutter(int t) {
  char key[6], hx[4];
  int i;

  sprintf(hx, "%04x", t);

  if (fd_socket >= 0) {
    key[0] = 's';
    for (i = 0; i < 4; i++) {
      key[1+i] = hx[i];
    }
    key[5] = '\0';

    write(fd_socket, key, sizeof(key));
    #ifdef DEBUG
    printf("# sending IICCD %s.\n", key);
    #endif
    if (fout != NULL) {
      fprintf(fout, "# changing shutter speed: sending IICCD %s.\n", key);
    }
    sleep(1);

    mseemon.time_iishutter = t;
    return(EXIT_SUCCESS);
  } else {
    printf("# finder is not connected.\n");
    return(EXIT_FAILURE);
  }
}


/* automatic start-up
 * v0.6.0 20041122 iwata */
int auto_startup(void) {
  int flg, ret;

  flg = 1;
  while (flg) {
    /* check dome status */
    read_finder_coords(); /* int socket_client.c */

    if (mseemon.domestatus == 1) {
      #ifdef DEBUG
      printf("# domestatus is %d.\n", mseemon.domestatus);
      #endif
      if (fout != NULL) {
	fprintf(fout, "# auto_startup: domestatus is %d.\n", mseemon.domestatus);
      }
      return(EXIT_FAILURE);
    } else if (mseemon.domestatus == 3) {
      flg = 0;
    } else {
      #ifdef DEBUG
      printf("# domestatus is %d.\n", mseemon.domestatus);
      #endif
      if (fout != NULL) {
	fprintf(fout, "# auto_startup: domestatus is %d.\n", mseemon.domestatus);
      }
      sleep(2);
    }    
  }

  /* read configuration file */
  /* added v0.6.9 20050111 */
  ret = read_config(PREFPATH); /* in prefs.c */
  if (ret == EXIT_FAILURE) {
    printf("# ERROR: failed in reading from %s.\n", PREFPATH);
    if (mseemon.autoflg == 0) {
      exit(EXIT_FAILURE);
    }
  }

  flg = 1;

  while (flg > 0) {
    /* telescope connection */
    ret = linkTel(); /* in telescope2.c */
    if (ret == EXIT_SUCCESS) {
      flg = 0;
      /* try waketel. added v0.5.11 20041107 iwata */
      sleep(2);
      wakeTel();
      /* update status text entry */
      update_tel_stat_entry();
      gtk_button_set_label(GTK_BUTTON(but_tel_sleep), "Sleep Telescope"); /* update button label; added v0.6.5 2004.12.07 */
    } else {
      flg++;
      /* try 5 times. if failed, return error */
      if (flg > 5) {
	printf("# tried to connect 5 times but failed. connection trouble?\n");
	if (fout != NULL) {
	  fprintf(fout, "# auto_startup: tried to connect to the telescope 5 times but failed. connection trouble?\n");
	}
	return(EXIT_FAILURE);
      } else {
	sleep(5);
      }
    }
  }

  /* open window */
  open_window_cap(); /* in cap_window_callbacks.c */

  mseemon.op_ssmode = 0;   /* auto star selection */
  mseemon.opmode = 1;      /* automatic operation mode */
  mseemon.is_pointing = 1; /* start from star selection */
  mseemon.tel_stat = 1;    /* connected, not pointed */
  pointing_proc(); /* telctl_automatic.c */

}



/* sleep for a while */
/* update v0.6.8 20050107 */
int long_sleep(int min) {
  int msec;
  char key[10];

  printf("# long_sleep(): sleeping %d minutes...\n", min);
  if (fout != NULL) {
    fprintf(fout, "# long_sleep(): sleeping %d minutes...\n", min);
  }

  stoptimer();
  reset_starfinding(); /* in cap_window_callbacks.c */
  gtk_widget_hide((GtkWidget*)window_capture);

  /* send dome close signal */
  if (fd_socket >= 0) {
      printf("# sending finder_comm a dome-close signal.\n");
      if (fout != NULL) {
	fprintf(fout, "# sending finder_comm a dome-close signal.\n");
      }
      strcpy(key, "dclose");
      write(fd_socket, key, sizeof(key));

      sleep(15);

      printf("# sending finder_comm CCD-off signal.\n");
      if (fout != NULL) {
	fprintf(fout, "# sending finder_comm CCD-off signal.\n");
      }
      strcpy(key, "ccdoff");
      write(fd_socket, key, sizeof(key));

      sleep(5);
  } else {
    if (fout != NULL) {
      fprintf(fout, "# unable to send finder_comm a dome-close signal, because socket is not connected.\n");
    }
  }

  update_main_displays(); /* in opmode.c */

  mseemon.is_sleeping = 1;
  mseemon.sleep_length = min;

  if (timertag_comm == -1) {
    start_comm_timer(); /* in command.c */
  }

  sleep(10);

  mseemon.is_pointing = 0;
  mseemon.is_capturing = 0;
  mseemon.is_guiding = 0; 

  mseemon.is_tracking = 0;
  mseemon.is_recording = 0;
}

/* resuming from a sleep caused by long_sleep() 
 * and start automatic obs. 20041208
 */
void resume_obs(void){
  char key[10];
  int ret;

  #ifdef DEBUG
  printf("# resume_obs: is_pointing=%d.\n", mseemon.is_pointing);
  #endif
  if (fout != NULL) {
    fprintf(fout, "# resume_obs: is_pointing=%d.\n", mseemon.is_pointing);
  }

  /*  check already restarted (manually) */
  if (mseemon.is_pointing == 0) {
    printf("# resuming observation...\n");
    if (fout != NULL) {
      fprintf(fout, "# resuming observation...\n");
    }
    mseemon.autoflg = 1; /* set auto mode true */

    /* check socket connection */
    if (fd_socket == -1) {
      if ((ret = connect_socket(socket_server)) == EXIT_SUCCESS) {
	printf("connected to %s.\n", socket_server);
      } else {
	if (fout != NULL) {
	  fprintf(fout, "# unable to connect to %s", socket_server);
	  fd_socket=-1;
	}
      }
    }

    /* time check, added v0.7.2 20050320 */
    ret = check_end_time();

    if (mseemon.opmode > 0) { // if time to stop op. has passed, opmode = 0
      if (fd_socket > -1) {
	/* check rain, humidity */
	read_finder_coords(); /* in socket_client.c */
	if (mseemon.rainflg == 0 &&
	    mseemon.humidity < HUM_LIMIT) {
	  if (mseemon.domestatus != 3) {
	    /* send dome open signal */
	    printf("# sending finder_comm a dome-open signal.\n");
	    if (fout != NULL) {
	      fprintf(fout, "# sending finder_comm a dome-open signal.\n");
	    }
	    strcpy(key, "dopen");
	    write(fd_socket, key, sizeof(key));
	  
	    printf("# sleeping 15sec.\n");
	    sleep(15);

	    /* send ccd-on signal */
	    strcpy(key, "ccdon");
	    write(fd_socket, key, sizeof(key));
	  
	    printf("# sleeping 15sec.\n");
	    sleep(15);

	    /* send finder start signal */
	    strcpy(key, "fstart");
	    write(fd_socket, key, sizeof(key));
	  
	    printf("# sleeping 180sec.\n");
	    sleep(180);
	  }

	  ret = auto_startup(); /* in telctl_automatic.c */

	  if (ret == EXIT_FAILURE) {
	    /* retry in next timeout */
	    mseemon.is_sleeping = mseemon.sleep_length;
	    #ifdef DEBUG
	    printf("# retry in next timeout.\n");
	    #endif
	    if (fout != NULL) {
	      fprintf(fout, "# resume_obs: retry in next timeout.\n");
	    }
	  }
	} else {
	  printf("# rainflg:%d humidity:%d.\n", mseemon.rainflg, mseemon.humidity);
	  printf("# sleeping again...\n");
	  if (fout != NULL) {
	    fprintf(fout, "# rainflg:%d humidity:%d.\n", mseemon.rainflg, mseemon.humidity);
	    fprintf(fout, "# sleeping again...\n");
	  }
	  /* long sleep again */
	  mseemon.is_sleeping = 1;
	}
      }
    }
  } else {
    if (fout != NULL) {
      fprintf(fout, "# resume_obs() called, but operation already resumed.\n");
    }
  }

}
