/* 
 * telescope control callback functions
 *
 * 2002.12.03 -
 * Time-stamp: <2004-11-07 18:30:29 dimm>
 *
 */
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <math.h>

#include "protocol.h"
#include "seemon2.h"

/* functions in protocol.c (added by Iwata; see also telescope2.c) */
extern void sleepTel();
extern void wakeTel();

/* update telescope status text entry */
void update_tel_stat_entry(void) {

  switch(mseemon.tel_stat) {
  case 0:
    gtk_entry_set_text(GTK_ENTRY(ent_tel_stat1), "disconnected");
    break;
  case 1:
    gtk_entry_set_text(GTK_ENTRY(ent_tel_stat1), "connected");
    break;
  case 2:
    gtk_entry_set_text(GTK_ENTRY(ent_tel_stat1), "pointing");
    break;
  case 3:
    gtk_entry_set_text(GTK_ENTRY(ent_tel_stat1), "pointed");
    break;
  case 4: /* added ver.0.4.6 */
    gtk_entry_set_text(GTK_ENTRY(ent_tel_stat1), "rest");
    break;
  default:
    gtk_entry_set_text(GTK_ENTRY(ent_tel_stat1), "unknown");
    break;
  }
}

/* connect/disconnect telescope */
void callback_connect_telescope(GtkWidget *widget, gpointer data)
{
  if (mseemon.tel_stat == 0) {
    linkTel(); /* in telescope2.c */
    /* try waketel. added v0.5.11 20041107 iwata */
    sleep(2);
    wakeTel();
  } else if (mseemon.tel_stat == 4) {
    /* telescope is sleeping. wake-up 
     * added v0.5.11 20041104 */
    wakeTel(); /* in telescope2.c */
    mseemon.tel_stat = 1;
    gtk_button_set_label(GTK_BUTTON(but_tel_sleep), "Sleep Telescope"); 
    sleep(2);
    linkTel(); /* in telescope2.c */
 } else {
    unlinkTel();
  }
  /* update status text entry */
  update_tel_stat_entry();
}

/* sleep/wake telescope; added ver.0.4.6 */
void callback_sleep_telescope(GtkWidget *widget, gpointer data) {
  GtkWidget* dialog;
  gint reply;

  if (mseemon.tel_stat == 4) {
    /* currently sleeping; so wake-up */
    wakeTel(); /* in telescope2.c */
    mseemon.tel_stat = 1;
    gtk_button_set_label(GTK_BUTTON(but_tel_sleep), "Sleep Telescope");
  } else {
    if (mseemon.tel_stat > 0 && mseemon.tel_stat < 4) {
      /* confirmation dialog. added ver.0.5.1 2003.10.01 */
      dialog = gtk_message_dialog_new(NULL, 
				      GTK_DIALOG_MODAL, 
				      GTK_MESSAGE_QUESTION,
				      GTK_BUTTONS_YES_NO, 
				      "Are you sure for sleeping telescope?");
      reply = gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
      #ifdef DEBUG
      printf("returned: %d\n", (int)reply);
      #endif
      if (reply == GTK_RESPONSE_YES) {
	#ifdef DEBUG
	printf("now going to sleep..\n");
        #endif
	/* currently wake-up; so go home and go to sleep */
	sleepTel();
	mseemon.tel_stat = 4;
	gtk_button_set_label(GTK_BUTTON(but_tel_sleep), "Wake-up Telescope");
      }
    }
  }
  /* update status text entry */
  update_tel_stat_entry();
}


/* point telescope 
 *  toward an edb entry */
void callback_point_telescope(GtkWidget *widget, gpointer data)
{
  int ret;
  Obj* cobj;

  if (mseemon.tel_stat < 1) {
    printf("telescope is disconnected.\n");
    return;
  }

  if (mseemon.selnum < 0) {
    printf("object is not selected.\n");
    return;
  } else {
    printf("pointing..\n");
    /* change tel_stat */
    mseemon.tel_stat = 2; /* pointing */
    update_tel_stat_entry();

    slewTel(); /* in telescope2.c */

    /* change tel_stat */
    mseemon.tel_stat = 3; /* pointed */
    update_tel_stat_entry();

    /* set telescope current pointing */
    cobj = obj_edb[mseemon.selnum];
    mseemon.tel_ra = cobj->s_ra;
    mseemon.tel_dc = cobj->s_dec;

    /* update telescope coord entries */
    showTelRA();
    showTelDec();
  }

}

/* point telescope 
 *  toward an edb entry 
 *  no gtk-callback ver.
 * 2003.08.16 */
void callback_point_telescope2(void)
{
  int ret;
  Obj* cobj;

  if (mseemon.tel_stat < 1) {
    printf("telescope is disconnected.\n");
    return;
  }

  if (mseemon.selnum < 0) {
    printf("object is not selected.\n");
    return;
  } else {
    printf("pointing..\n");
    /* change tel_stat */
    mseemon.tel_stat = 2; /* pointing */
    update_tel_stat_entry();

    slewTel(); /* in telescope2.c */

    /* change tel_stat */
    mseemon.tel_stat = 3; /* pointed */
    update_tel_stat_entry();

    /* set telescope current pointing */
    cobj = obj_edb[mseemon.selnum];
    mseemon.tel_ra = cobj->s_ra;
    mseemon.tel_dc = cobj->s_dec;

    /* update telescope coord entries */
    showTelRA();
    showTelDec();
  }

}


/* rebuild message for goto command */
void build_msg (void) {
  Obj* cobj;
  int i;
  unsigned char s, t;
  int p, q;
  char r;
  int rah, ram, dcd, dcm;
  double ras, dcs;
  double tra, tdc; /* ra in hms, dec in deg */
  char buf[256], sbuf[256];
  char *tbuf;

  s = 0;
  tra = mseemon.tel_ra * 12./PI;
  tdc = mseemon.tel_dc * 180./PI;

  cobj = obj_edb[mseemon.selnum];

  /* create msg for goto command */
  strncpy(buf, "user_def,f|S|O0,", 16);
  s = 16;

  /* R.A. */
  rah = (int)(tra);
  ram = (int)((tra - rah) * 60.);
  ras = ((tra - rah) * 60. - ram) * 60.;

  sprintf(sbuf, "%d", rah);
  p = strlen(sbuf);
  for (i = 0; i < p; i++) {
    buf[s] = sbuf[i];
    s++;
  }
  buf[s++] = ':';
  sprintf(sbuf, "%d", ram);
  p = strlen(sbuf);
  for (i = 0; i < p; i++) {
    buf[s] = sbuf[i];
    s++;
  }
  buf[s++] = ':';

  tbuf = fcvt(ras, 2, &p, &q);

  /*
  #ifdef DEBUG
  printf("%f %s %d\n", ras, tbuf, p);
  #endif
  */

  for (i = 0; i < p; i++) {
    buf[s++] = tbuf[i];
  }
  buf[s++] = '.';
  for (i = 0; i < 2; i++) {
    buf[s++] = tbuf[p+i];
  }
  buf[s++] = ',';

  /* Decl */
  if (tdc >= 0) {
    dcd = (int)(tdc);
    dcm = (int)((tdc - dcd) * 60.);
    dcs = ((tdc - dcd) * 60. - dcm) * 60.;
  } else {
    dcd = (int)(-1.*tdc);
    dcm = (int)((-1.*tdc - dcd) * 60.);
    dcs = ((-1*tdc - dcd) * 60. - dcm) * 60.;
    buf[s++] = '-';
  }
  sprintf(sbuf, "%d", dcd);
  p = strlen(sbuf);
  for (i = 0; i < p; i++) {
    buf[s] = sbuf[i];
    s++;
  }
  buf[s++] = ':';
  sprintf(sbuf, "%d", dcm);
  p = strlen(sbuf);
  for (i = 0; i < p; i++) {
    buf[s] = sbuf[i];
    s++;
  }
  buf[s++] = ':';

  tbuf = fcvt(dcs, 1, &p, &q);
  for (i = 0; i < p; i++) {
    buf[s++] = tbuf[i];
  }
  buf[s++] = '.';
  buf[s++] = tbuf[p];

  strcpy(sbuf, ",99.9,2000");
  for (i = 0; i < 10; i++) {
    buf[s++] = sbuf[i];
  }
  buf[s] = '\0';

  /* copy to msg in mseemon struct */
  strcpy(mseemon.msg, buf);
  //  #ifdef DEBUG
  //  printf ("rebuilt message: %s\n", buf);
  //  #endif
}

/* callback functions 
 * for "handbox"
 */
/* button is pressed */
void callback_tel_ctl(GtkWidget *widget, gpointer data) {
  int i;
  int *dir = data;
  int teldir;

  #ifdef DEBUG
  printf("button pressed. dir=%d\n", *dir);
  #endif

  switch(*dir) {
    /* N */
  case 1: 
    teldir=NORTH;
    StartSlew(teldir);
    break;

    /* E */
  case 2: 
    teldir=EAST;
    StartSlew(teldir);
    break;
    
    /* W */
  case 3:
    teldir=WEST;
    StartSlew(teldir);              
    break;
       
    /* S */
  case 4: 
    teldir=SOUTH;
    StartSlew(teldir);       
    break;
  }

}

void callback_tel_ctlstop(GtkWidget *widget, gpointer data) {
  int i;
  int *dir = data;
  int teldir;

  #ifdef DEBUG
  printf("button released. dir=%d\n", *dir);
  #endif

  switch(*dir) {
    /* N */
  case 1: 
    teldir=NORTH;
    StopSlew(teldir);
    break;

    /* E */
  case 2: 
    teldir=EAST;
    StopSlew(teldir);
    break;

    /* W */
  case 3:
    teldir=WEST;
    StopSlew(teldir);              
    break;

    /* S */
  case 4: 
    teldir=SOUTH;
    StopSlew(teldir);       
    break;
  }

}


/* callback functions for setting step */
void callback_tel_setstep1(GtkWidget *widget, gpointer data) {
  #ifdef DEBUG
  printf("current: %p widget: %p\n", toggle_cstep, widget);
  #endif

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
      /* If control reaches here, the toggle button is down */
      if (data != (gpointer)toggle_cstep) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_cstep), FALSE);
	mseemon.tel_step = GUIDE;
	toggle_cstep = (GtkWidget*)data;
	/* execute step change */
	//	SetRate(mseemon.tel_step);
	/*********************************/
	/*test */
	SetRateCenter0();
	mseemon.tel_step = CENTER; /* added by iwata v0.5.10 20040810 */
      }
    }
}

void callback_tel_setstep2(GtkWidget *widget, gpointer data) {
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
      /* If control reaches here, the toggle button is down */
      if (data != (gpointer)toggle_cstep) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_cstep), FALSE);
	mseemon.tel_step = CENTER;
	toggle_cstep = (GtkWidget*)data;
	/* execute step change */
	SetRate(mseemon.tel_step);
      }
    }
}

void callback_tel_setstep3(GtkWidget *widget, gpointer data) {
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
      /* If control reaches here, the toggle button is down */
      if (data != (gpointer)toggle_cstep) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_cstep), FALSE);
	mseemon.tel_step = FIND;
	toggle_cstep = (GtkWidget*)data;
	/* execute step change */
	SetRate(mseemon.tel_step);
      }
    }
}

void callback_tel_setstep4(GtkWidget *widget, gpointer data) {
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
      /* If control reaches here, the toggle button is down */
      if (data != (gpointer)toggle_cstep) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_cstep), FALSE);
	mseemon.tel_step = SLEW;
	toggle_cstep = (GtkWidget*)data;
	/* execute step change */
	SetRate(mseemon.tel_step);
      }
    }
}

/* focus control. v0.5.0 2003.09.11 */
void callback_focus_pos(GtkWidget *widget, gpointer data)
{
  StartFocusPos();
}

void callback_focus_neg(GtkWidget *widget, gpointer data)
{
  StartFocusNeg();
}

void callback_focusstop(GtkWidget *widget, gpointer data)
{
  StopFocus();
}
