#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <math.h>
#include <gtk/gtk.h>

#include "seemon2.h"

/*
 * get current time
 */
int get_time(myTime *ttime) {
  int now, yr;
  int d, t, i, flg1;

  int mday[13] = {0,31,28,31,30,31,30,31,31,30,31,30,31};

  /* 2002.01.01 */
  /* '8' is for leap years */
  d = (int)(((2001-1970)*365 + 8 ) * 24 *3600);
  now = time(NULL);

  now -= d;
  t = now;

//#ifdef DEBUG
//  printf("%d\n", now);
//#endif

  yr = 2001;
  
  i = 1;
  flg1 = 1;
  while (flg1) {
    t -= mday[i] * 24 * 3600;
    if (t < 0) {
      flg1 = 0;
    } else {
      i++;
      if (i > 12) {
	yr++;
	i = 1;
	if ((yr%4) == 0) {
	  /* leap year */
	  mday[2] = 29;
	} else {
	  mday[2] = 28;
	}
      }
      now = t;
    }
  }

  ttime->year  = yr;
  ttime->month = i;                   /* month */
  ttime->day   = now / (24*3600) + 1; /* day */
  now -= (ttime->day-1) * 24 * 3600;
  ttime->hour  = now / 3600;          /* hour */
  now -= ttime->hour * 3600;
  ttime->min   = now / 60;            /* min */
  now -= ttime->min * 60;
  ttime->sec   = now;                 /* sec */

  //  printf("%d %d %d %d %d\n", yr, ttime->month, ttime->day, ttime->hour, ttime->min, ttime->sec);

  return(EXIT_SUCCESS);
}


/* get current local time */
int get_localtime(myTime *ttime) {
  int ret;
  ret = get_time(ttime);
  ttime->hour += (int)(msite.lt);
  return(ret);
}

/* get current modified julian day */
double myCalcmjd(int yy, int mm, int dd, int hr, int mn) {
  double ret;
  int y, m;

  if (mm <= 2) {
    y = yy - 1;
    m = mm + 12;
  } else {
    y = yy;
    m = mm;
  }
  
  ret = (int)(365.25 * y) + (int)(y / 400) - (int)(y / 100) ;
  ret += (int)(30.59 * ( m - 2)) + dd - 678912;
  ret += (hr + msite.lt) / 24. + mn / 1440. - 0.375;
  
  return ret;
}


/* get current lst */
double get_lst(double tmjd, double lon) {
  double d, ret;

  d = (0.671262 + 1.0027379094 * (tmjd - 40000) + lon/360);
  ret = (d - (int)(d)) * 24.;

  return ret;
}


/* get az-el for an obj */
int get_azel(int onum, double lst) {
  double ha, s, azm, alt;
  double slat;

  /* convert lst to radian */
  lst = lst * PI / 12.0;
  slat = msite.slat * PI / 180;

  /* calc hour angle */
  ha = lst - obj_edb[onum]->s_ra;

  /* calc altitude */
  s = cos(obj_edb[onum]->s_dec) * cos(ha) * cos(slat) +
    sin(obj_edb[onum]->s_dec) * sin(slat);
  alt = asin(s);

  /* calc azimuth */
  s = (cos(obj_edb[onum]->s_dec) * sin(ha)) / cos(alt);
  azm = asin(s);
  
  obj_edb[onum]->s_az  = azm;
  obj_edb[onum]->s_alt = alt;
  obj_edb[onum]->s_ha  = ha;

  return(EXIT_SUCCESS);
}

/* get elevation and hour angle for a ra-dec pair */
int get_alt(double ra, double dec, double *alt, double *ha) {
  myTime mt;
  //  double ha, s, azm, alt;
  double s, azm;
  double d, tmjd, lst;
  double slat;

  //  #ifdef DEBUG
  //  printf("# RA:%5.3f Dec:%5.3f\n", ra*180./PI, dec*180./PI);
  //  #endif

  get_time(&mt);
  d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
  tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
  lst = get_lst(tmjd, msite.slon);
  /* convert lst to radian */
  lst = lst * PI / 12.0;
  slat = msite.slat * PI / 180;

  /* calc hour angle */
  *ha = lst - ra;

  /* calc altitude */
  s = cos(dec) * cos(*ha) * cos(slat) + sin(dec) * sin(slat);
  *alt = asin(s);

  /* calc azimuth */
  /*
  s = cos(dec) * sin(*ha) / cos(alt);
  azm = asin(s);
  */

  return(EXIT_SUCCESS);
}



/* 
 * from libastro/mjd.c 
 */

/* given a date in months, mn, days, dy, years, yr,
 * return the modified Julian date (number of days elapsed since 1900 jan 0.5),
 * *mjd.
 */
void
cal_mjd (mn, dy, yr, tmjd)
int mn, yr;
double dy;
double *tmjd;
{
	static double last_mjd, last_dy;
	static int last_mn, last_yr;
	int b, d, m, y;
	long c;

	if (mn == last_mn && yr == last_yr && dy == last_dy) {
	    *tmjd = last_mjd;
	    return;
	}

	m = mn;
	y = (yr < 0) ? yr + 1 : yr;
	if (mn < 3) {
	    m += 12;
	    y -= 1;
	}

	if (yr < 1582 || (yr == 1582 && (mn < 10 || (mn == 10 && dy < 15))))
	    b = 0;
	else {
	    int a;
	    a = y/100;
	    b = 2 - a + a/4;
	}

	if (y < 0)
	    c = (long)((365.25*y) - 0.75) - 694025L;
	else
	    c = (long)(365.25*y) - 694025L;

	d = (int)(30.6001*(m+1));

	*tmjd = b + c + d + dy - 0.5;

	last_mn = mn;
	last_dy = dy;
	last_yr = yr;
	last_mjd = *tmjd;
}

/* given the modified Julian date (number of days elapsed since 1900 jan 0.5,),
 * mjd, return the calendar date in months, *mn, days, *dy, and years, *yr.
 */
void
mjd_cal (tmjd, mn, dy, yr)
double tmjd;
int *mn, *yr;
double *dy;
{
	static double last_mjd, last_dy;
	static int last_mn, last_yr;
	double d, f;
	double i, a, b, ce, g;

	/* we get called with 0 quite a bit from unused epoch fields.
	 * 0 is noon the last day of 1899.
	 */
	if (tmjd == 0.0) {
	    *mn = 12;
	    *dy = 31.5;
	    *yr = 1899;
	    return;
	}

	if (tmjd == last_mjd) {
	    *mn = last_mn;
	    *yr = last_yr;
	    *dy = last_dy;
	    return;
	}

	d = tmjd + 0.5;
	i = floor(d);
	f = d-i;
	if (f == 1) {
	    f = 0;
	    i += 1;
	}

	if (i > -115860.0) {
	    a = floor((i/36524.25)+.99835726)+14;
	    i += 1 + a - floor(a/4.0);
	}

	b = floor((i/365.25)+.802601);
	ce = i - floor((365.25*b)+.750001)+416;
	g = floor(ce/30.6001);
	*mn = (int)(g - 1);
	*dy = ce - floor(30.6001*g)+f;
	*yr = (int)(b + 1899);

	if (g > 13.5)
	    *mn = (int)(g - 13);
	if (*mn < 2.5)
	    *yr = (int)(b + 1900);
	if (*yr < 1)
	    *yr -= 1;

	last_mn = *mn;
	last_dy = *dy;
	last_yr = *yr;
	last_mjd = tmjd;
}


/* given a mjd, return the year as a double. */
void mjd_year (tmjd, yr)
double tmjd;
double *yr;
{
	static double last_mjd, last_yr;
	int m, y;
	double d;
	double e0, e1;	/* mjd of start of this year, start of next year */

	if (tmjd == last_mjd) {
	    *yr = last_yr;
	    return;
	}

	mjd_cal (tmjd, &m, &d, &y);
	if (y == -1) y = -2;
	cal_mjd (1, 1.0, y, &e0);
	cal_mjd (1, 1.0, y+1, &e1);
	*yr = y + (tmjd - e0)/(e1 - e0);

	last_mjd = tmjd;
	last_yr = *yr;
}

/* given an mjd, truncate it to the beginning of the whole day */
double
mjd_day(jd)
double jd;
{
  return (floor(jd-0.5)+0.5);
}





/* for testing
int main(void) {
  myTime t;
  double d, tmjd, lst;

  /* oao 
  msite.slon = 133.59667; /* 8h 54m 23.2s, +23d 24m 26s 
  msite.slat = 23.407222;
  msite.lt = 9;

  get_time(&t);
  d = t.day + t.hour / 24. + t.min / (24.*60.) + t.sec / (24.*3600.);
  //cal_mjd(t.month, d, t.year, &tmjd);
  tmjd =  myCalcmjd(t.year, t.month, t.day, t.hour, t.min);
  lst = get_lst(tmjd, msite.slon);

  printf("%4d %2d %2d %2d %2d %2d\n", t.year, t.month, t.day, 
	 t.hour, t.min, t.sec);
  printf("%f %f\n", tmjd, lst);

  return(0);
}
*/

